@@ -42,7 +42,8 @@ class TrackSegment : public osg::Object
 public:
     TrackSegment() {}
 
-    TrackSegment(const TrackSegment& ts, const osg::CopyOp=osg::CopyOp::SHALLOW_COPY) {}
+    TrackSegment(const TrackSegment& ts, const osg::CopyOp copyop=osg::CopyOp::SHALLOW_COPY):
+        osg::Object(ts, copyop) {}
 
     META_Object(osg, TrackSegment)
 
@@ -78,7 +79,8 @@ class Track : public osg::Object
 public:
     Track() {}
 
-    Track(const Track& track, const osg::CopyOp=osg::CopyOp::SHALLOW_COPY) {}
+    Track(const Track& track, const osg::CopyOp copyop=osg::CopyOp::SHALLOW_COPY):
+        osg::Object(track, copyop) {}
 
     META_Object(osg, Track)
 
@@ -437,11 +439,11 @@ int main(int argv, char **argc)
     osg::ref_ptr<osg::Node> loadedModel = osgDB::readRefNodeFiles(arguments);
     if (loadedModel.valid()) group->addChild(loadedModel);
 
-    for(Tracks::iterator itr = tracks.begin();
-        itr != tracks.end();
-        ++itr)
+    for(Tracks::iterator titr = tracks.begin();
+        titr != tracks.end();
+        ++titr)
     {
-        Track* track = itr->get();
+        Track* track = titr->get();
 
         group->addChild(createTrackModel(track, osg::Vec4(1.0,1.0,1.0,1.0)));
 
@@ -547,11 +549,11 @@ int main(int argv, char **argc)
 
         fout<<"<?xml version=\"1.0\" encoding=\"utf-8\"?><gpx version=\"1.0\" creator=\"osggpx\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns=\"http://www.topografix.com/GPX/1/0\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">"<<std::endl;
 
-        for(Tracks::iterator itr = tracks.begin();
-            itr != tracks.end();
-            ++itr)
+        for(Tracks::iterator titr = tracks.begin();
+            titr != tracks.end();
+            ++titr)
         {
-            Track* track = itr->get();
+            Track* track = titr->get();
 
             fout<<"<trk>"<<std::endl;
             fout<<"<desc>The track description</desc>"<<std::endl;
