@@ -438,7 +438,7 @@ int main(int argc, char** argv)
     if (arguments.read("-devices") || arguments.read("--devices"))
     {
         // Force load QuickTime plugin, probe video capability, exit
-        osgDB::readImageFile("devices.live");
+        osgDB::readRefImageFile("devices.live");
         return 1;
     }
 #endif
@@ -641,7 +641,11 @@ int main(int argc, char** argv)
         if (wsi)
         {
             unsigned int width, height;
-            wsi->getScreenResolution(osg::GraphicsContext::ScreenIdentifier(0), width, height);
+            osg::GraphicsContext::ScreenIdentifier main_screen_id;
+
+            main_screen_id.readDISPLAY();
+            main_screen_id.setUndefinedScreenDetailsToDefaultScreen();
+            wsi->getScreenResolution(main_screen_id, width, height);
 
             screenAspectRatio = float(width) / float(height);
         }
@@ -727,8 +731,8 @@ void SDLAudioSink::play()
 
 
 
-    SDL_AudioSpec specs = { 0 };
-    SDL_AudioSpec wanted_specs = { 0 };
+    SDL_AudioSpec specs;
+    SDL_AudioSpec wanted_specs;
 
     wanted_specs.freq = _audioStream->audioFrequency();
     wanted_specs.channels = _audioStream->audioNbChannels();
