@@ -16,7 +16,7 @@
 *  THE SOFTWARE.
 */
 
-#ifdef WIN32
+#if defined(WIN32) && !(defined(__CYGWIN__) || defined(__MINGW32__))
 /////////////////////////////////////////////////////////////////////////////
 // Disable unavoidable warning messages:
 
@@ -395,7 +395,7 @@ int main(int argc, char **argv)
                 csn->removeChildren(0, csn->getNumChildren());
                 csn->addChild(overlayNode.get());
 
-                // tell the overlay node to continously update its overlay texture
+                // tell the overlay node to continuously update its overlay texture
                 // as we know we'll be tracking a moving target.
                 overlayNode->setContinuousUpdate(true);
             }
