@@ -103,12 +103,12 @@ struct PopulateHistogram
 
     void luminance(float l) { update(l); }
     void alpha(float a) { update(a); }
-    void luminance_alpha(float l, float a) { update(l); }
-    void rgb(float r, float g, float b) { update(r); }
-    void rgba(float r, float g, float b, float a) { update(a); }
+    void luminance_alpha(float l, float /*a*/) { update(l); }
+    void rgb(float r, float /*g*/, float /*b*/) { update(r); }
+    void rgba(float /*r*/, float /*g*/, float /*b*/, float a) { update(a); }
 };
 
-void Histogram::analyse(const osg::Image* image, double interval)
+void Histogram::analyse(const osg::Image* image, double /*interval*/)
 {
     PopulateHistogram populateHistogram(_valueMap);
     readImage(image, populateHistogram);
@@ -407,13 +407,12 @@ int main(int argc, char ** argv)
     osg::ArgumentParser arguments(&argc, argv);
 
 #if 0
-
     osg::ref_ptr<MyClass> myobject = new MyClass;
     myobject->getOrCreateUserDataContainer()->addUserObject(new osg::CallbackObject("myMethod"));
     myobject->myMethod();
 
-    osg::ref_ptr<osg::ScriptEngine> se = osgDB::readFile<osg::ScriptEngine>("ScriptEngine.lua");
-    osg::ref_ptr<osg::Script> script = osgDB::readFile<osg::Script>("script.lua");
+    osg::ref_ptr<osg::ScriptEngine> se = osgDB::readRefFile<osg::ScriptEngine>("ScriptEngine.lua");
+    osg::ref_ptr<osg::Script> script = osgDB::readRefFile<osg::Script>("script.lua");
 
     osg::ref_ptr<MyClass> copyobject = new MyClass;
     copyobject->getOrCreateUserDataContainer()->addUserObject(new MyScriptCallback(se.get(), script.get(), "myMethod"));
@@ -421,7 +420,7 @@ int main(int argc, char ** argv)
 #endif
 
 #if 0
-    osg::ref_ptr<osg::Object> object = osgDB::readNodeFile("load.lua");
+    osg::ref_ptr<osg::Object> object = osgDB::readRefNodeFile("load.lua");
     if (object.valid())
     {
         osg::CallbackObject* co = osg::getCallbackObject(object.get(), "method");
@@ -463,57 +462,8 @@ int main(int argc, char ** argv)
 
     bool createHistorgram = arguments.read("--histogram");
 
-#if 0
-    for(int i=1; i<arguments.argc(); ++i)
-    {
-        if (!arguments.isOption(i))
-        {
-            osg::ref_ptr<osg::Image> image;
-            osg::ref_ptr<osgVolume::Volume> volume;
-            osg::ref_ptr<osgVolume::VolumeTile> volumeTile;
-
-            std::string filename = arguments[i];
-            osgDB::FileType fileType = osgDB::fileType(foundFile);
-
-            if (fileType == osgDB::DIRECTORY)
-            {
-                osg::ref_ptr<osg::Image> image = osgDB::readImageFile(foundFile+".dicom", options.get());
-            }
-            else if (fileType == osgDB::REGULAR_FILE)
-            {
-                std::string ext = osgDB::getFileExtension(foundFile);
-                if (ext=="osg" || ext=="ive" || ext=="osgx" || ext=="osgb" || ext=="osgt")
-                {
-                    osg::ref_ptr<osg::Object> obj = osgDB::readObjectFile(foundFile);
-                    image = dynamic_cast<osg::Image*>(obj.get());
-                    volume = dynamic_cast<osgVolume::Volume*>(obj.get());
-                    volumeTile = dynamic_cast<osgVolume::VolumeTile*>(obj.get());
-                }
-                else
-                {
-                    image = osgDB::readImageFile( foundFile );
-                }
-            }
-            else
-            {
-                // not found image, so fallback to plugins/callbacks to find the model.
-                image = osgDB::readImageFile( filename);
-            }
-
-            if (image.valid())
-            {
-                volumeTile = new osgVolume::VolumeTile;
-            }
-
-        }
-        OSG_NOTICE<<"Argument "<<i<<" "<<arguments[i]<<std::endl;
-    }
-
-    return 1;
-#else
-
     osg::ref_ptr<osg::Node> model = osgDB::readRefNodeFiles(arguments);
-#endif
+
     typedef std::vector< osg::ref_ptr<osg::Node> > Nodes;
     Nodes nodes;
 
