@@ -31,9 +31,6 @@
     #include <sys/unistd.h>
 #endif
 #endif
-#if defined(__sgi)
-#include <unistd.h>
-#endif
 #if defined(__hpux)
 #include <sys/mpctl.h>
 #endif
@@ -67,6 +64,7 @@ using namespace OpenThreads;
 # define DPRINTF(arg)
 #endif
 
+
 //-----------------------------------------------------------------------------
 // Initialize the static unique ids.
 //
@@ -111,6 +109,56 @@ void thread_cleanup_handler(void *arg)
 namespace OpenThreads
 {
 
+#if defined(HAVE_PTHREAD_SETAFFINITY_NP) || defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY) || defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
+static void setAffinity(const Affinity& affinity)
+{
+    //std::cout<<"setProcessAffinity : "<< affinity.activeCPUs.size() <<std::endl;
+    cpu_set_t cpumask;
+    CPU_ZERO( &cpumask );
+    unsigned int numprocessors = OpenThreads::GetNumberOfProcessors();
+    if (affinity)
+    {
+        for(Affinity::ActiveCPUs::const_iterator itr = affinity.activeCPUs.begin();
+            itr != affinity.activeCPUs.end();
+            ++itr)
+        {
+            if (*itr<numprocessors)
+            {
+                //std::cout<<"   setting CPU : "<< *itr<<std::endl;
+                CPU_SET( *itr, &cpumask );
+            }
+        }
+    }
+    else
+    {
+        // BUG-fix for linux:
+        // Each thread inherits the processor affinity mask from its parent thread.
+        // We need to explicitly set it to all CPUs, if no affinity was specified.
+        for (unsigned int i = 0; i < numprocessors; ++i)
+        {
+            //std::cout<<"   Fallback setting CPU : "<< i<<std::endl;
+
+            CPU_SET( i, &cpumask );
+        }
+    }
+
+#if defined(HAVE_PTHREAD_SETAFFINITY_NP)
+        pthread_setaffinity_np( pthread_self(), sizeof(cpumask), &cpumask);
+#elif defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY)
+        sched_setaffinity( 0, sizeof(cpumask), &cpumask );
+#elif defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
+        sched_setaffinity( 0, &cpumask );
+#endif
+
+}
+#else
+static void setAffinity(const Affinity&)
+{
+// No supported.
+}
+#endif
+
+
 class ThreadPrivateActions
 {
 
@@ -132,49 +180,8 @@ private:
         PThreadPrivateData *pd =
         static_cast<PThreadPrivateData *>(thread->_prvData);
 
-
-        if (pd->cpunum>=0)
-        {
-#if defined(__sgi)
-            pthread_setrunon_np( pd->cpunum );
-#elif defined(HAVE_PTHREAD_SETAFFINITY_NP) || defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY) || defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-            cpu_set_t cpumask;
-            CPU_ZERO( &cpumask );
-            CPU_SET( pd->cpunum, &cpumask );
-
-#if defined(HAVE_PTHREAD_SETAFFINITY_NP)
-            pthread_setaffinity_np( pthread_self(), sizeof(cpumask), &cpumask);
-#elif defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY)
-            sched_setaffinity( 0, sizeof(cpumask), &cpumask );
-#elif defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-            sched_setaffinity( 0, &cpumask );
-#endif
-#endif
-        }
-#if defined(HAVE_PTHREAD_SETAFFINITY_NP) || defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY) || defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-        else
-        {
-            // BUG-fix for linux:
-            // Each thread inherits the processor affinity mask from its parent thread.
-            // We need to explicitly set it to all CPUs, if no affinity was specified.
-
-            cpu_set_t cpumask;
-            CPU_ZERO( &cpumask );
-
-            for (int i = 0; i < OpenThreads::GetNumberOfProcessors(); ++i)
-            {
-                CPU_SET( i, &cpumask );
-            }
-
-#if defined(HAVE_PTHREAD_SETAFFINITY_NP)
-            pthread_setaffinity_np( pthread_self(), sizeof(cpumask), &cpumask);
-#elif defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY)
-            sched_setaffinity( 0, sizeof(cpumask), &cpumask );
-#elif defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-            sched_setaffinity( 0, &cpumask );
-#endif
-        }
-#endif
+        // set up processor affinity
+        setAffinity( pd->affinity );
 
         ThreadCleanupStruct tcs;
         tcs.thread = thread;
@@ -184,7 +191,7 @@ private:
         int status = pthread_setspecific(PThreadPrivateData::s_tls_key, thread);
         if (status)
         {
-            printf("Error: pthread_setspecific(,) returned error status, status = %d\n",status);
+           printf("Error: pthread_setspecific(,) returned error status, status = %d\n",status);
         }
 
         pthread_cleanup_push(thread_cleanup_handler, &tcs);
@@ -232,12 +239,12 @@ private:
 
             if(status != 0) {
             printf("THREAD INFO (%d) : Get sched: %s\n",
-                   thread->getProcessId(),
+                   (int)thread->getProcessId(),
                    strerror(status));
             } else {
             printf(
                 "THREAD INFO (%d) : Thread running at %s / Priority: %d\n",
-                thread->getProcessId(),
+                (int)thread->getProcessId(),
                 (my_policy == SCHED_FIFO ? "SCHEDULE_FIFO"
                  : (my_policy == SCHED_RR ? "SCHEDULE_ROUND_ROBIN"
                 : (my_policy == SCHED_OTHER ? "SCHEDULE_OTHER"
@@ -249,9 +256,8 @@ private:
 
             printf(
                 "THREAD INFO (%d) : Max priority: %d, Min priority: %d\n",
-                thread->getProcessId(),
+                (int)thread->getProcessId(),
                 max_priority, min_priority);
-
             }
 
         }
@@ -259,7 +265,7 @@ private:
         {
             printf(
             "THREAD INFO (%d) POSIX Priority scheduling not available\n",
-            thread->getProcessId());
+            (int)thread->getProcessId());
         }
 
         fflush(stdout);
@@ -288,8 +294,6 @@ private:
             pthread_getschedparam(thread->getProcessId(),
                       &th_policy, &th_param);
 
-#ifndef __linux__
-
             switch(thread->getSchedulePolicy())
             {
 
@@ -306,32 +310,14 @@ private:
                 break;
 
                 default:
-#ifdef __sgi
-                th_policy = SCHED_RR;
-#else
                 th_policy = SCHED_FIFO;
-#endif
                 break;
             };
 
-#else
-            th_policy = SCHED_OTHER;  // Must protect linux from realtime.
-#endif
-
-#ifdef __linux__
-
-            max_priority = 0;
-            min_priority = 20;
-            nominal_priority = (max_priority + min_priority)/2;
-
-#else
-
             max_priority = sched_get_priority_max(th_policy);
             min_priority = sched_get_priority_min(th_policy);
             nominal_priority = (max_priority + min_priority)/2;
 
-#endif
-
             switch(thread->getSchedulePriority())
             {
 
@@ -415,7 +401,7 @@ int Thread::GetConcurrency()
 
 //----------------------------------------------------------------------------
 //
-// Decription: Constructor
+// Description: Constructor
 //
 // Use: public.
 //
@@ -425,16 +411,6 @@ Thread::Thread()
     if(!s_isInitialized) Init();
 
     PThreadPrivateData *pd = new PThreadPrivateData();
-    pd->stackSize = 0;
-    pd->stackSizeLocked = false;
-    pd->idSet = false;
-    pd->setRunning(false);
-    pd->isCanceled = false;
-    pd->uniqueId = pd->nextId;
-    pd->nextId++;
-    pd->threadPriority = Thread::THREAD_PRIORITY_DEFAULT;
-    pd->threadPolicy = Thread::THREAD_SCHEDULE_DEFAULT;
-    pd->cpunum = -1;
 
     _prvData = static_cast<void *>(pd);
 
@@ -442,7 +418,7 @@ Thread::Thread()
 
 //----------------------------------------------------------------------------
 //
-// Decription: Destructor
+// Description: Destructor
 //
 // Use: public.
 //
@@ -568,52 +544,41 @@ size_t Thread::getProcessId()
     return (size_t)(pd->tid);
 }
 
+int OpenThreads::SetProcessorAffinityOfCurrentThread(const Affinity& affinity)
+{
+    Thread::Init();
+
+    Thread* thread = Thread::CurrentThread();
+    if (thread)
+    {
+        return thread->setProcessorAffinity(affinity);
+    }
+    else
+    {
+        // set up processor affinity
+        setAffinity( affinity );
+    }
+
+    return -1;
+}
+
 //-----------------------------------------------------------------------------
 //
 // Description: Set the thread's processor affinity
 //
 // Use: public
 //
-int Thread::setProcessorAffinity(unsigned int cpunum)
+int Thread::setProcessorAffinity(const Affinity& affinity)
 {
     PThreadPrivateData *pd = static_cast<PThreadPrivateData *> (_prvData);
-    pd->cpunum = cpunum;
-    if (pd->cpunum<0) return -1;
-
-#ifdef __sgi
-
-    int status;
-    pthread_attr_t thread_attr;
-
-    status = pthread_attr_init( &thread_attr );
-    if(status != 0)
-    {
-        return status;
-    }
-
-    status = pthread_attr_setscope( &thread_attr, PTHREAD_SCOPE_BOUND_NP );
-    return status;
-
-#elif defined(HAVE_PTHREAD_SETAFFINITY_NP) || defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY) || defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
+    pd->affinity = affinity;
 
     if (pd->isRunning() && Thread::CurrentThread()==this)
     {
-        cpu_set_t cpumask;
-        CPU_ZERO( &cpumask );
-        CPU_SET( pd->cpunum, &cpumask );
-#if defined(HAVE_PTHREAD_SETAFFINITY_NP)
-        return pthread_setaffinity_np (pthread_self(), sizeof(cpumask), &cpumask);
-#elif defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY)
-        return sched_setaffinity( 0, sizeof(cpumask), &cpumask );
-#elif defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-        return sched_setaffinity( 0, &cpumask );
-#endif
+        setAffinity( affinity );
     }
 
     return -1;
-#else
-    return -1;
-#endif
 
 }
 
@@ -638,6 +603,12 @@ bool Thread::isRunning()
 //
 int Thread::start() {
 
+    PThreadPrivateData *pd = static_cast<PThreadPrivateData *> (_prvData);
+    if (pd->isRunning())
+    {
+        return 0;
+    }
+
     int status;
     pthread_attr_t thread_attr;
 
@@ -647,8 +618,6 @@ int Thread::start() {
         return status;
     }
 
-    PThreadPrivateData *pd = static_cast<PThreadPrivateData *> (_prvData);
-
     //-------------------------------------------------------------------------
     // Set the stack size if requested, but not less than a platform reasonable
     // value.
@@ -659,7 +628,7 @@ int Thread::start() {
         if(pd->stackSize < PTHREAD_STACK_MIN)
             pd->stackSize = PTHREAD_STACK_MIN;
 #endif
-        pthread_attr_setstacksize( &thread_attr, pd->stackSize);
+        status = pthread_attr_setstacksize( &thread_attr, pd->stackSize);
         if(status != 0)
         {
             return status;
@@ -670,7 +639,7 @@ int Thread::start() {
     // Now get what we actually have...
     //
     size_t size;
-    pthread_attr_getstacksize( &thread_attr, &size);
+    status = pthread_attr_getstacksize( &thread_attr, &size);
     if(status != 0)
     {
         return status;
@@ -687,15 +656,16 @@ int Thread::start() {
     status = pthread_attr_setinheritsched( &thread_attr,
                        PTHREAD_EXPLICIT_SCHED );
 
-    pthread_attr_setscope(&thread_attr, PTHREAD_SCOPE_SYSTEM);
-
-#endif // ] ALLOW_PRIORITY_SCHEDULING
-
     if(status != 0)
     {
         return status;
     }
 
+    pthread_attr_setscope(&thread_attr, PTHREAD_SCOPE_SYSTEM);
+
+#endif // ] ALLOW_PRIORITY_SCHEDULING
+
+
     pd->threadStartedBlock.reset();
 
     status = pthread_create(&(pd->tid), &thread_attr,
@@ -862,7 +832,7 @@ int Thread::setSchedulePriority(ThreadPriority priority) {
 
     pd->threadPriority = priority;
 
-    if(pd->isRunning)
+    if(pd->isRunning())
         return ThreadPrivateActions::SetThreadSchedulingParams(this);
     else
         return 0;
@@ -902,7 +872,7 @@ int Thread::setSchedulePolicy(ThreadPolicy policy)
 
     pd->threadPolicy = policy;
 
-    if(pd->isRunning)
+    if(pd->isRunning())
     return ThreadPrivateActions::SetThreadSchedulingParams(this);
     else
     return 0;
@@ -1024,11 +994,6 @@ int OpenThreads::GetNumberOfProcessors()
    if (ret == -1)
       return 0;
    return ret;
-#elif defined(__sgi)
-   long ret = sysconf(_SC_NPROC_ONLN);
-   if (ret == -1)
-      return 0;
-   return ret;
 #elif defined(__hpux)
    int ret = mpctl(MPC_GETNUMSPUS, 0, NULL);
    if (ret == -1)
@@ -1048,33 +1013,3 @@ int OpenThreads::GetNumberOfProcessors()
 #endif
 }
 
-int OpenThreads::SetProcessorAffinityOfCurrentThread(unsigned int cpunum)
-{
-    Thread::Init();
-
-    Thread* thread = Thread::CurrentThread();
-    if (thread)
-    {
-        return thread->setProcessorAffinity(cpunum);
-    }
-    else
-    {
-#if defined(HAVE_PTHREAD_SETAFFINITY_NP) || defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY) || defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-        cpu_set_t cpumask;
-        CPU_ZERO( &cpumask );
-        CPU_SET( cpunum, &cpumask );
-#if defined(HAVE_PTHREAD_SETAFFINITY_NP)
-        pthread_setaffinity_np( pthread_self(), sizeof(cpumask), &cpumask);
-        return 0;
-#elif defined(HAVE_THREE_PARAM_SCHED_SETAFFINITY)
-        sched_setaffinity( 0, sizeof(cpumask), &cpumask );
-        return 0;
-#elif defined(HAVE_TWO_PARAM_SCHED_SETAFFINITY)
-        sched_setaffinity( 0, &cpumask );
-        return 0;
-#endif
-#endif
-    }
-
-    return -1;
-}
