@@ -25,12 +25,21 @@ ColorMaski::~ColorMaski()
 {
 }
 
+void ColorMaski::setIndex(unsigned int buf)
+{
+    if (_index==buf) return;
+
+    ReassignToParents needToReassingToParentsWhenMemberValueChanges(this);
+
+    _index = buf;
+}
+
 void ColorMaski::apply(State& state) const
 {
     const GLExtensions* extensions = state.get<GLExtensions>();
     if (extensions->glColorMaski)
     {
-        extensions->glColorMaski((GLboolean)_index, (GLboolean)_red,(GLboolean)_green,(GLboolean)_blue,(GLboolean)_alpha);
+        extensions->glColorMaski((GLuint)_index, (GLboolean)_red,(GLboolean)_green,(GLboolean)_blue,(GLboolean)_alpha);
     }
     else
     {
