@@ -17,6 +17,8 @@
 #include <osg/Camera>
 #include <osg/View>
 #include <osg/GLObjects>
+#include <osg/ContextData>
+#include <osg/os_utils>
 
 #include <osg/FrameBufferObject>
 #include <osg/Program>
@@ -33,42 +35,92 @@
 #include <sstream>
 #include <algorithm>
 #include <iterator>
+#include <stdio.h>
 
 using namespace osg;
 
 /////////////////////////////////////////////////////////////////////////////
+//
+// WindowSystemInterfaces
+//
+GraphicsContext::WindowingSystemInterfaces::WindowingSystemInterfaces()
+{
+}
 
+GraphicsContext::WindowingSystemInterfaces::~WindowingSystemInterfaces()
+{
+}
+
+void GraphicsContext::WindowingSystemInterfaces::addWindowingSystemInterface(GraphicsContext::WindowingSystemInterface* wsi)
+{
+    if (std::find(_interfaces.begin(), _interfaces.end(), wsi)==_interfaces.end())
+    {
+        _interfaces.push_back(wsi);
+    }
+}
+
+void GraphicsContext::WindowingSystemInterfaces::removeWindowingSystemInterface(GraphicsContext::WindowingSystemInterface* wsi)
+{
+    Interfaces::iterator itr = std::find(_interfaces.begin(), _interfaces.end(), wsi);
+    if (itr!=_interfaces.end())
+    {
+        _interfaces.erase(itr);
+    }
+}
+
+GraphicsContext::WindowingSystemInterface* GraphicsContext::WindowingSystemInterfaces::getWindowingSystemInterface(const std::string& name)
+{
+    if (_interfaces.empty())
+    {
+        OSG_WARN<<"Warning: GraphicsContext::WindowingSystemInterfaces::getWindowingSystemInterface() failed, no interfaces available."<<std::endl;
+        return 0;
+    }
+
+    if (!name.empty())
+    {
+        for(Interfaces::iterator itr = _interfaces.begin();
+            itr != _interfaces.end();
+            ++itr)
+        {
+            if ((*itr)->getName()==name)
+            {
+                return itr->get();
+            }
+
+            OSG_NOTICE<<"   tried interface "<<typeid(*itr).name()<<", name= "<<(*itr)->getName()<<std::endl;
+        }
+
+        OSG_WARN<<"Warning: GraphicsContext::WindowingSystemInterfaces::getWindowingSystemInterface() failed, no interfaces matches name : "<<name<<std::endl;
+        return 0;
+    }
+    else
+    {
+        // no preference provided so just take the first available interface
+        return _interfaces.front().get();
+    }
+}
 
 // Use a static reference pointer to hold the window system interface.
 // Wrap this within a function, in order to control the order in which
 // the static pointer's constructor is executed.
-
-static ref_ptr<GraphicsContext::WindowingSystemInterface> &windowingSystemInterfaceRef()
+osg::ref_ptr<GraphicsContext::WindowingSystemInterfaces>& GraphicsContext::getWindowingSystemInterfaces()
 {
-    static ref_ptr<GraphicsContext::WindowingSystemInterface> s_WindowingSystemInterface;
+    static ref_ptr<GraphicsContext::WindowingSystemInterfaces> s_WindowingSystemInterface = new GraphicsContext::WindowingSystemInterfaces;
     return s_WindowingSystemInterface;
 }
 
+OSG_INIT_SINGLETON_PROXY(ProxyInitWindowingSystemInterfaces, GraphicsContext::getWindowingSystemInterfaces())
+
 
 //  GraphicsContext static method implementations
-
-void GraphicsContext::setWindowingSystemInterface(WindowingSystemInterface* callback)
+GraphicsContext::WindowingSystemInterface* GraphicsContext::getWindowingSystemInterface(const std::string& name)
 {
-    ref_ptr<GraphicsContext::WindowingSystemInterface> &wsref = windowingSystemInterfaceRef();
-    wsref = callback;
-    OSG_INFO<<"GraphicsContext::setWindowingSystemInterface() "<<wsref.get()<<"\t"<<&wsref<<std::endl;
-}
-
-GraphicsContext::WindowingSystemInterface* GraphicsContext::getWindowingSystemInterface()
-{
-    ref_ptr<GraphicsContext::WindowingSystemInterface> &wsref = windowingSystemInterfaceRef();
-    OSG_INFO<<"GraphicsContext::getWindowingSystemInterface() "<<wsref.get()<<"\t"<<&wsref<<std::endl;
-    return wsref.get();
+    return GraphicsContext::getWindowingSystemInterfaces()->getWindowingSystemInterface(name);
 }
 
 GraphicsContext* GraphicsContext::createGraphicsContext(Traits* traits)
 {
-    ref_ptr<GraphicsContext::WindowingSystemInterface> &wsref = windowingSystemInterfaceRef();
+    ref_ptr<GraphicsContext::WindowingSystemInterface> wsref = getWindowingSystemInterface(traits ? traits->windowingSystemPreference : "") ;
     if ( wsref.valid())
     {
         // catch any undefined values.
@@ -102,10 +154,10 @@ std::string GraphicsContext::ScreenIdentifier::displayName() const
 
 void GraphicsContext::ScreenIdentifier::readDISPLAY()
 {
-    const char* ptr = 0;
-    if ((ptr=getenv("DISPLAY")) != 0)
+    std::string str;
+    if (getEnvVar("DISPLAY", str))
     {
-        setScreenIdentifier(ptr);
+        setScreenIdentifier(str);
     }
 }
 
@@ -187,7 +239,7 @@ GraphicsContext::Traits::Traits(DisplaySettings* ds):
             swapBarrier(0),
             useMultiThreadedOpenGLEngine(false),
             useCursor(true),
-            glContextVersion("1.0"),
+            glContextVersion(OSG_GL_CONTEXT_VERSION),
             glContextFlags(0),
             glContextProfileMask(0),
             sharedContext(0),
@@ -232,7 +284,7 @@ bool GraphicsContext::Traits::getContextVersion(unsigned int& major, unsigned in
     return true;
 }
 
-
+#if 0
 class ContextData
 {
 public:
@@ -261,197 +313,63 @@ public:
     osg::ref_ptr<osg::GraphicsContext> _compileContext;
 
 };
-
-
-typedef std::map<unsigned int, ContextData>  ContextIDMap;
-static ContextIDMap s_contextIDMap;
-static OpenThreads::ReentrantMutex s_contextIDMapMutex;
-static GraphicsContext::GraphicsContexts s_registeredContexts;
+#endif
 
 unsigned int GraphicsContext::createNewContextID()
 {
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-
-    // first check to see if we can reuse contextID;
-    for(ContextIDMap::iterator itr = s_contextIDMap.begin();
-        itr != s_contextIDMap.end();
-        ++itr)
-    {
-        if (itr->second._numContexts == 0)
-        {
-
-            // reuse contextID;
-            itr->second._numContexts = 1;
-
-            OSG_INFO<<"GraphicsContext::createNewContextID() reusing contextID="<<itr->first<<std::endl;
-
-            return itr->first;
-        }
-    }
-
-    unsigned int contextID = s_contextIDMap.size();
-    s_contextIDMap[contextID]._numContexts = 1;
-
-    OSG_INFO<<"GraphicsContext::createNewContextID() creating contextID="<<contextID<<std::endl;
-    OSG_INFO<<"Updating the MaxNumberOfGraphicsContexts to "<<contextID+1<<std::endl;
-
-    // update the maximum number of graphics contexts,
-    // to ensure that texture objects and display buffers are configured to the correct size.
-    osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts( contextID + 1 );
-
-    return contextID;
+    return ContextData::createNewContextID();
 }
 
 unsigned int GraphicsContext::getMaxContextID()
 {
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    unsigned int maxContextID = 0;
-    for(ContextIDMap::iterator itr = s_contextIDMap.begin();
-        itr != s_contextIDMap.end();
-        ++itr)
-    {
-        if (itr->first > maxContextID) maxContextID = itr->first;
-    }
-    return maxContextID;
+    return ContextData::getMaxContextID();
 }
 
 
 void GraphicsContext::incrementContextIDUsageCount(unsigned int contextID)
 {
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-
-    s_contextIDMap[contextID].incrementUsageCount();
-
-    OSG_INFO<<"GraphicsContext::incrementContextIDUsageCount("<<contextID<<") to "<<s_contextIDMap[contextID]._numContexts<<std::endl;
+    return ContextData::incrementContextIDUsageCount(contextID);
 }
 
 void GraphicsContext::decrementContextIDUsageCount(unsigned int contextID)
 {
-
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-
-    if (s_contextIDMap[contextID]._numContexts!=0)
-    {
-        s_contextIDMap[contextID].decrementUsageCount();
-    }
-    else
-    {
-        OSG_NOTICE<<"Warning: decrementContextIDUsageCount("<<contextID<<") called on expired contextID."<<std::endl;
-    }
-
-    OSG_INFO<<"GraphicsContext::decrementContextIDUsageCount("<<contextID<<") to "<<s_contextIDMap[contextID]._numContexts<<std::endl;
-
+    return ContextData::decrementContextIDUsageCount(contextID);
 }
 
 
 void GraphicsContext::registerGraphicsContext(GraphicsContext* gc)
 {
-    OSG_INFO<<"GraphicsContext::registerGraphicsContext "<<gc<<std::endl;
-
-    if (!gc) return;
-
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-
-    GraphicsContexts::iterator itr = std::find(s_registeredContexts.begin(), s_registeredContexts.end(), gc);
-    if (itr != s_registeredContexts.end()) s_registeredContexts.erase(itr);
-
-    s_registeredContexts.push_back(gc);
+    ContextData::registerGraphicsContext(gc);
 }
 
 void GraphicsContext::unregisterGraphicsContext(GraphicsContext* gc)
 {
-    OSG_INFO<<"GraphicsContext::unregisterGraphicsContext "<<gc<<std::endl;
-
-    if (!gc) return;
-
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-
-    GraphicsContexts::iterator itr = std::find(s_registeredContexts.begin(), s_registeredContexts.end(), gc);
-    if (itr != s_registeredContexts.end()) s_registeredContexts.erase(itr);
+    ContextData::unregisterGraphicsContext(gc);
 }
 
 GraphicsContext::GraphicsContexts GraphicsContext::getAllRegisteredGraphicsContexts()
 {
-    OSG_INFO<<"GraphicsContext::getAllRegisteredGraphicsContexts s_registeredContexts.size()="<<s_registeredContexts.size()<<std::endl;
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    return s_registeredContexts;
+    return ContextData::getAllRegisteredGraphicsContexts();
 }
 
 GraphicsContext::GraphicsContexts GraphicsContext::getRegisteredGraphicsContexts(unsigned int contextID)
 {
-    GraphicsContexts contexts;
-
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    for(GraphicsContexts::iterator itr = s_registeredContexts.begin();
-        itr != s_registeredContexts.end();
-        ++itr)
-    {
-        GraphicsContext* gc = *itr;
-        if (gc->getState() && gc->getState()->getContextID()==contextID) contexts.push_back(gc);
-    }
-
-    OSG_INFO<<"GraphicsContext::getRegisteredGraphicsContexts "<<contextID<<" contexts.size()="<<contexts.size()<<std::endl;
-
-    return contexts;
+    return ContextData::getRegisteredGraphicsContexts(contextID);
 }
 
 GraphicsContext* GraphicsContext::getOrCreateCompileContext(unsigned int contextID)
 {
-    OSG_NOTICE<<"GraphicsContext::createCompileContext."<<std::endl;
-
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-        if (s_contextIDMap[contextID]._compileContext.valid()) return s_contextIDMap[contextID]._compileContext.get();
-    }
-
-    GraphicsContext::GraphicsContexts contexts = GraphicsContext::getRegisteredGraphicsContexts(contextID);
-    if (contexts.empty()) return 0;
-
-    GraphicsContext* src_gc = contexts.front();
-    const osg::GraphicsContext::Traits* src_traits = src_gc->getTraits();
-
-    osg::GraphicsContext::Traits* traits = new osg::GraphicsContext::Traits;
-    traits->screenNum = src_traits->screenNum;
-    traits->displayNum = src_traits->displayNum;
-    traits->hostName = src_traits->hostName;
-    traits->width = 100;
-    traits->height = 100;
-    traits->red = src_traits->red;
-    traits->green = src_traits->green;
-    traits->blue = src_traits->blue;
-    traits->alpha = src_traits->alpha;
-    traits->depth = src_traits->depth;
-    traits->sharedContext = src_gc;
-    traits->pbuffer = true;
-
-    osg::ref_ptr<osg::GraphicsContext> gc = osg::GraphicsContext::createGraphicsContext(traits);
-    if (gc.valid() && gc->realize())
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-        s_contextIDMap[contextID]._compileContext = gc;
-        OSG_NOTICE<<"   succeeded GraphicsContext::createCompileContext."<<std::endl;
-        return gc.release();
-    }
-    else
-    {
-        return 0;
-    }
-
+    return ContextData::getOrCreateCompileContext(contextID);
 }
 
 void GraphicsContext::setCompileContext(unsigned int contextID, GraphicsContext* gc)
 {
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    s_contextIDMap[contextID]._compileContext = gc;
+    return ContextData::setCompileContext(contextID, gc);
 }
 
 GraphicsContext* GraphicsContext::getCompileContext(unsigned int contextID)
 {
-    // OSG_NOTICE<<"GraphicsContext::getCompileContext "<<contextID<<std::endl;
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-    ContextIDMap::iterator itr = s_contextIDMap.find(contextID);
-    if (itr != s_contextIDMap.end()) return itr->second._compileContext.get();
-    else return 0;
+    return ContextData::getCompileContext(contextID);
 }
 
 
@@ -520,7 +438,7 @@ bool GraphicsContext::realize()
 
 void GraphicsContext::close(bool callCloseImplementation)
 {
-    OSG_INFO<<"close("<<callCloseImplementation<<")"<<this<<std::endl;
+    OSG_INFO<<"GraphicsContext::close("<<callCloseImplementation<<")"<<this<<std::endl;
 
     // switch off the graphics thread...
     setGraphicsThread(0);
@@ -530,8 +448,8 @@ void GraphicsContext::close(bool callCloseImplementation)
 
     if (_state.valid())
     {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_contextIDMapMutex);
-        if (s_contextIDMap[_state->getContextID()]._numContexts>1) sharedContextExists = true;
+        osg::ContextData* cd = osg::getContextData(_state->getContextID());
+        if (cd && cd->getNumContexts()>1) sharedContextExists = true;
     }
 
     // release all the OpenGL objects in the scene graphs associated with this
@@ -651,7 +569,7 @@ void GraphicsContext::swapBuffers()
 {
     if (isCurrent())
     {
-        swapBuffersCallbackOrImplemenation();
+        swapBuffersCallbackOrImplementation();
         clear();
     }
     else if (_graphicsThread.valid() &&
@@ -662,7 +580,7 @@ void GraphicsContext::swapBuffers()
     else
     {
         makeCurrent();
-        swapBuffersCallbackOrImplemenation();
+        swapBuffersCallbackOrImplementation();
         clear();
     }
 }
@@ -672,6 +590,11 @@ void GraphicsContext::createGraphicsThread()
     if (!_graphicsThread)
     {
         setGraphicsThread(new GraphicsThread);
+
+        if (_traits.valid())
+        {
+            _graphicsThread->setProcessorAffinity(_traits->affinity);
+        }
     }
 }
 
@@ -850,7 +773,7 @@ void GraphicsContext::removeCamera(osg::Camera* camera)
             nitr != nodes.end();
             ++nitr)
         {
-            const_cast<osg::Node*>(*nitr)->releaseGLObjects(_state.get());
+            (*nitr)->releaseGLObjects(_state.get());
         }
 
         // release the context of the any RenderingCache that the Camera has.
@@ -987,6 +910,7 @@ void GraphicsContext::resizedImplementation(int x, int y, int width, int height)
 SyncSwapBuffersCallback::SyncSwapBuffersCallback()
 {
     OSG_INFO<<"Created SyncSwapBuffersCallback."<<std::endl;
+    _previousSync = 0;
 }
 
 void SyncSwapBuffersCallback::swapBuffersImplementation(osg::GraphicsContext* gc)
@@ -1013,3 +937,4 @@ void SyncSwapBuffersCallback::swapBuffersImplementation(osg::GraphicsContext* gc
 
     //OSG_NOTICE<<"After swap"<<std::endl;
 }
+
