@@ -32,6 +32,7 @@
 #include <osg/ColorMask>
 #include <osg/PolygonOffset>
 #include <osg/Depth>
+#include <osg/ContextData>
 #include <map>
 #include <vector>
 
@@ -97,15 +98,19 @@ StateSet* initOQDebugState()
 
 struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
 {
-    typedef std::vector<osg::TestResult*> ResultsVector;
+    typedef std::vector<osg::ref_ptr<osg::TestResult> > ResultsVector;
     ResultsVector _results;
 
-    RetrieveQueriesCallback( osg::GLExtensions* ext=NULL )
-      : _extensionsFallback( ext )
+    RetrieveQueriesCallback( osg::GLExtensions* ext=NULL )  :
+        _extensionsFallback( ext )
+    {
+    }
+
+    RetrieveQueriesCallback( const RetrieveQueriesCallback& rqc, const osg::CopyOp& ) :
+        _extensionsFallback( rqc._extensionsFallback )
     {
     }
 
-    RetrieveQueriesCallback( const RetrieveQueriesCallback&, const osg::CopyOp& ) {}
     META_Object( osgOQ, RetrieveQueriesCallback )
 
     virtual void operator() (const osg::Camera& camera) const
@@ -143,7 +148,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
         ResultsVector::const_iterator it = _results.begin();
         while (it != _results.end())
         {
-            osg::TestResult* tr = const_cast<osg::TestResult*>( *it );
+            osg::TestResult* tr = const_cast<osg::TestResult*>( (*it).get() );
 
             if (!tr->_active || !tr->_init)
             {
@@ -159,31 +164,20 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
             OSG_DEBUG <<
                 "osgOQ: RQCB: Retrieving..." << std::endl;
 
-#ifdef FORCE_QUERY_RESULT_AVAILABLE_BEFORE_RETRIEVAL
-
-            // Should not need to do this, but is required on some platforms to
-            // work aroung issues in the device driver. For example, without this
-            // code, we've seen crashes on 64-bit Mac/Linux NVIDIA systems doing
-            // multithreaded, multipipe rendering (as in a CAVE).
-            // Tried with ATI and verified this workaround is not needed; the
-            //   problem is specific to NVIDIA.
-            GLint ready( 0 );
-            while( !ready )
+            GLint ready = 0;
+            ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT_AVAILABLE, &ready );
+            if (ready)
             {
-                // Apparently, must actually sleep here to avoid issues w/ NVIDIA Quadro.
-                OpenThreads::Thread::microSleep( 5 );
-                ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT_AVAILABLE, &ready );
-            };
-#endif
+                ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT, &(tr->_numPixels) );
+                if (tr->_numPixels < 0)
+                    OSG_WARN << "osgOQ: RQCB: " <<
+                    "glGetQueryObjectiv returned negative value (" << tr->_numPixels << ")." << std::endl;
 
-            ext->glGetQueryObjectiv( tr->_id, GL_QUERY_RESULT, &(tr->_numPixels) );
-            if (tr->_numPixels < 0)
-                OSG_WARN << "osgOQ: RQCB: " <<
-                "glGetQueryObjectiv returned negative value (" << tr->_numPixels << ")." << std::endl;
-
-            // Either retrieve last frame's results, or ignore it because the
-            //   camera is inside the view. In either case, _active is now false.
-            tr->_active = false;
+                // Either retrieve last frame's results, or ignore it because the
+                //   camera is inside the view. In either case, _active is now false.
+                tr->_active = false;
+            }
+            // else: query result not available yet, try again next frame
 
             it++;
             count++;
@@ -196,7 +190,13 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
 
     void reset()
     {
-        _results.clear();
+        for (ResultsVector::iterator it = _results.begin(); it != _results.end();)
+        {
+            if (!(*it)->_active || !(*it)->_init) // remove results that have already been retrieved or their query objects deleted.
+                it = _results.erase(it);
+            else
+                ++it;
+        }
     }
 
     void add( osg::TestResult* tr )
@@ -213,7 +213,7 @@ struct RetrieveQueriesCallback : public osg::Camera::DrawCallback
 struct ClearQueriesCallback : public osg::Camera::DrawCallback
 {
     ClearQueriesCallback() : _rqcb( NULL ) {}
-    ClearQueriesCallback( const ClearQueriesCallback&, const osg::CopyOp& ) {}
+    ClearQueriesCallback( const ClearQueriesCallback& rhs, const osg::CopyOp& copyop) : osg::Camera::DrawCallback(rhs, copyop), _rqcb(rhs._rqcb) {}
     META_Object( osgOQ, ClearQueriesCallback )
 
     virtual void operator() (const osg::Camera&) const
@@ -230,18 +230,22 @@ struct ClearQueriesCallback : public osg::Camera::DrawCallback
 };
 
 
-// static cache of deleted query objects which can only
-// be completely deleted once the appropriate OpenGL context
-// is set.
-typedef std::list< GLuint > QueryObjectList;
-typedef osg::buffered_object< QueryObjectList > DeletedQueryObjectCache;
-
-static OpenThreads::Mutex s_mutex_deletedQueryObjectCache;
-static DeletedQueryObjectCache s_deletedQueryObjectCache;
 
 namespace osg
 {
 
+class QueryObjectManager : public GLObjectManager
+{
+public:
+    QueryObjectManager(unsigned int contextID) : GLObjectManager("QueryObjectManager", contextID) {}
+
+    virtual void deleteGLObject(GLuint globj)
+    {
+        const GLExtensions* extensions = GLExtensions::Get(_contextID,true);
+        if (extensions->isOcclusionQuerySupported || extensions->isARBOcclusionQuerySupported) extensions->glDeleteQueries( 1L, &globj );
+    }
+};
+
 
 QueryGeometry::QueryGeometry( const std::string& oqnName )
   : _oqnName( oqnName )
@@ -264,9 +268,9 @@ QueryGeometry::reset()
     ResultMap::iterator it = _results.begin();
     while (it != _results.end())
     {
-        TestResult& tr = it->second;
-        if (tr._init)
-            QueryGeometry::deleteQueryObject( tr._contextID, tr._id );
+        osg::ref_ptr<TestResult> tr = it->second;
+        if (tr->_init)
+            QueryGeometry::deleteQueryObject( tr->_contextID, tr->_id );
         it++;
     }
     _results.clear();
@@ -280,6 +284,10 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
 {
     unsigned int contextID = renderInfo.getState()->getContextID();
     osg::GLExtensions* ext = renderInfo.getState()->get<GLExtensions>();
+
+    if (!ext->isARBOcclusionQuerySupported && !ext->isOcclusionQuerySupported)
+        return;
+
     osg::Camera* cam = renderInfo.getCurrentCamera();
 
     // Add callbacks if necessary.
@@ -294,10 +302,30 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
     }
 
     // Get TestResult from Camera map
-    TestResult* tr;
+    osg::ref_ptr<osg::TestResult> tr;
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _mapMutex );
-        tr = &( _results[ cam ] );
+        tr = ( _results[ cam ] );
+        if (!tr.valid())
+        {
+            tr = new osg::TestResult;
+            _results[ cam ] = tr;
+        }
+    }
+
+
+    // Issue query
+    if (!tr->_init)
+    {
+        ext->glGenQueries( 1, &(tr->_id) );
+        tr->_contextID = contextID;
+        tr->_init = true;
+    }
+
+    if (tr->_active)
+    {
+        // last query hasn't been retrieved yet
+        return;
     }
 
     // Add TestResult to RQCB.
@@ -308,16 +336,7 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
         OSG_FATAL << "osgOQ: QG: Invalid RQCB." << std::endl;
         return;
     }
-    rqcb->add( tr );
-
-
-    // Issue query
-    if (!tr->_init)
-    {
-        ext->glGenQueries( 1, &(tr->_id) );
-        tr->_contextID = contextID;
-        tr->_init = true;
-    }
+    rqcb->add( tr.get() );
 
     OSG_DEBUG <<
         "osgOQ: QG: Querying for: " << _oqnName << std::endl;
@@ -345,22 +364,32 @@ QueryGeometry::drawImplementation( osg::RenderInfo& renderInfo ) const
 
 }
 
+QueryGeometry::QueryResult QueryGeometry::getQueryResult( const osg::Camera* cam )
+{
+    osg::ref_ptr<osg::TestResult> tr;
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _mapMutex );
+        tr =  _results[ cam ];
+        if (!tr.valid())
+        {
+            tr = new osg::TestResult;
+            _results[ cam ] = tr;
+        }
+    }
+    return QueryResult((tr->_init && !tr->_active), tr->_numPixels);
+}
 
 unsigned int
 QueryGeometry::getNumPixels( const osg::Camera* cam )
 {
-    TestResult tr;
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _mapMutex );
-        tr =  _results[ cam ];
-    }
-    return tr._numPixels;
+    return getQueryResult(cam).numPixels;
 }
 
-
 void
 QueryGeometry::releaseGLObjects( osg::State* state ) const
 {
+    Geometry::releaseGLObjects(state);
+
     if (!state)
     {
         // delete all query IDs for all contexts.
@@ -375,11 +404,11 @@ QueryGeometry::releaseGLObjects( osg::State* state ) const
         ResultMap::iterator it = _results.begin();
         while (it != _results.end())
         {
-            TestResult& tr = it->second;
-            if (tr._contextID == contextID)
+            osg::ref_ptr<osg::TestResult> tr = it->second;
+            if (tr->_contextID == contextID)
             {
-                QueryGeometry::deleteQueryObject( contextID, tr._id );
-                tr._init = false;
+                osg::get<QueryObjectManager>(contextID)->scheduleGLObjectForDeletion(tr->_id );
+                tr->_init = false;
             }
             it++;
         }
@@ -389,52 +418,20 @@ QueryGeometry::releaseGLObjects( osg::State* state ) const
 void
 QueryGeometry::deleteQueryObject( unsigned int contextID, GLuint handle )
 {
-    if (handle!=0)
-    {
-        OpenThreads::ScopedLock< OpenThreads::Mutex > lock( s_mutex_deletedQueryObjectCache );
-
-        // insert the handle into the cache for the appropriate context.
-        s_deletedQueryObjectCache[contextID].push_back( handle );
-    }
+    osg::get<QueryObjectManager>(contextID)->scheduleGLObjectForDeletion(handle);
 }
 
 
 void
-QueryGeometry::flushDeletedQueryObjects( unsigned int contextID, double /*currentTime*/, double& availableTime )
+QueryGeometry::flushDeletedQueryObjects( unsigned int contextID, double currentTime, double& availableTime )
 {
-    // if no time available don't try to flush objects.
-    if (availableTime<=0.0) return;
-
-    const osg::Timer& timer = *osg::Timer::instance();
-    osg::Timer_t start_tick = timer.tick();
-    double elapsedTime = 0.0;
-
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedQueryObjectCache);
-
-        const osg::GLExtensions* extensions = osg::GLExtensions::Get( contextID, true );
-
-        QueryObjectList& qol = s_deletedQueryObjectCache[contextID];
-
-        for(QueryObjectList::iterator titr=qol.begin();
-            titr!=qol.end() && elapsedTime<availableTime;
-            )
-        {
-            extensions->glDeleteQueries( 1L, &(*titr ) );
-            titr = qol.erase(titr);
-            elapsedTime = timer.delta_s(start_tick,timer.tick());
-        }
-    }
-
-    availableTime -= elapsedTime;
+    osg::get<QueryObjectManager>(contextID)->flushDeletedGLObjects(currentTime, availableTime);
 }
 
 void
 QueryGeometry::discardDeletedQueryObjects( unsigned int contextID )
 {
-    OpenThreads::ScopedLock< OpenThreads::Mutex > lock( s_mutex_deletedQueryObjectCache );
-    QueryObjectList& qol = s_deletedQueryObjectCache[ contextID ];
-    qol.clear();
+    osg::get<QueryObjectManager>(contextID)->discardAllGLObjects();
 }
 
 // End support classes
@@ -445,6 +442,8 @@ QueryGeometry::discardDeletedQueryObjects( unsigned int contextID )
 
 OcclusionQueryNode::OcclusionQueryNode()
   : _enabled( true ),
+    _validQueryGeometry( false ),
+    _passed(false),
     _visThreshold( 500 ),
     _queryFrameCount( 5 ),
     _debugBB( false )
@@ -461,6 +460,7 @@ OcclusionQueryNode::~OcclusionQueryNode()
 
 OcclusionQueryNode::OcclusionQueryNode( const OcclusionQueryNode& oqn, const CopyOp& copyop )
   : Group( oqn, copyop ),
+    _validQueryGeometry( false ),
     _passed( false )
 {
     _enabled = oqn._enabled;
@@ -476,9 +476,27 @@ OcclusionQueryNode::OcclusionQueryNode( const OcclusionQueryNode& oqn, const Cop
 bool OcclusionQueryNode::getPassed( const Camera* camera, NodeVisitor& nv )
 {
     if ( !_enabled )
+    {
         // Queries are not enabled. The caller should be osgUtil::CullVisitor,
         //   return true to traverse the subgraphs.
-        return true;
+        _passed = true;
+        return _passed;
+    }
+
+    QueryGeometry* qg = static_cast< QueryGeometry* >( _queryGeode->getDrawable( 0 ) );
+
+    if ( !_validQueryGeometry )
+    {
+        // There're cases that the occlusion test result has been retrieved
+        // after the query geometry has been changed, it's the result of the
+        // geometry before the change.
+        qg->reset();
+
+        // The box of the query geometry is invalid, return false to not traverse
+        // the subgraphs.
+        _passed = false;
+        return _passed;
+    }
 
     {
         // Two situations where we want to simply do a regular traversal:
@@ -489,19 +507,22 @@ bool OcclusionQueryNode::getPassed( const Camera* camera, NodeVisitor& nv )
         const unsigned int& lastQueryFrame( _frameCountMap[ camera ] );
         if( ( lastQueryFrame == 0 ) ||
             ( (nv.getTraversalNumber() - lastQueryFrame) >  (_queryFrameCount + 1) ) )
-            return true;
+        {
+            _passed = true;
+            return _passed;
+        }
     }
 
     if (_queryGeode->getDrawable( 0 ) == NULL)
     {
         OSG_FATAL << "osgOQ: OcclusionQueryNode: No QueryGeometry." << std::endl;
         // Something's broke. Return true so we at least render correctly.
-        return true;
+        _passed = true;
+        return _passed;
     }
-    QueryGeometry* qg = static_cast< QueryGeometry* >( _queryGeode->getDrawable( 0 ) );
 
     // Get the near plane for the upcoming distance calculation.
-    float nearPlane;
+    osg::Matrix::value_type nearPlane;
     const osg::Matrix& proj( camera->getProjectionMatrix() );
     if( ( proj(3,3) != 1. ) || ( proj(2,3) != 0. ) || ( proj(1,3) != 0. ) || ( proj(0,3) != 0.) )
         nearPlane = proj(3,2) / (proj(2,2)-1.);  // frustum / perspective
@@ -512,14 +533,22 @@ bool OcclusionQueryNode::getPassed( const Camera* camera, NodeVisitor& nv )
     //   the results. Otherwise (near plane inside the BS shell) we are considered
     //   to have passed and don't need to retrieve the query.
     const osg::BoundingSphere& bs = getBound();
-    float distanceToEyePoint = nv.getDistanceToEyePoint( bs._center, false );
+    osg::Matrix::value_type distanceToEyePoint = nv.getDistanceToEyePoint( bs._center, false );
 
-    float distance = distanceToEyePoint - nearPlane - bs._radius;
-    _passed = ( distance <= 0.f );
+    osg::Matrix::value_type distance = distanceToEyePoint - nearPlane - bs._radius;
+    _passed = ( distance <= 0.0 );
     if (!_passed)
     {
-        int result = qg->getNumPixels( camera );
-        _passed = ( (unsigned int)(result) > _visThreshold );
+        QueryGeometry::QueryResult result = qg->getQueryResult( camera );
+        if (!result.valid)
+        {
+           // The query hasn't finished yet and the result still
+           // isn't available, return true to traverse the subgraphs.
+           _passed = true;
+           return _passed;
+        }
+
+        _passed = ( result.numPixels > _visThreshold );
     }
 
     return _passed;
@@ -527,6 +556,9 @@ bool OcclusionQueryNode::getPassed( const Camera* camera, NodeVisitor& nv )
 
 void OcclusionQueryNode::traverseQuery( const Camera* camera, NodeVisitor& nv )
 {
+    if (!_validQueryGeometry || ! _enabled)
+        return;
+
     bool issueQuery;
     {
         const int curFrame = nv.getTraversalNumber();
@@ -543,9 +575,11 @@ void OcclusionQueryNode::traverseQuery( const Camera* camera, NodeVisitor& nv )
 
 void OcclusionQueryNode::traverseDebug( NodeVisitor& nv )
 {
-    if (_debugBB)
+    if (_debugBB && _enabled)
+    {
         // If requested, display the debug geometry
         _debugGeode->accept( nv );
+    }
 }
 
 BoundingSphere OcclusionQueryNode::computeBound() const
@@ -564,17 +598,28 @@ BoundingSphere OcclusionQueryNode::computeBound() const
         ComputeBoundsVisitor cbv;
         nonConstThis->accept( cbv );
         BoundingBox bb = cbv.getBoundingBox();
+        const bool bbValid = bb.valid();
+        _validQueryGeometry = bbValid;
 
         osg::ref_ptr<Vec3Array> v = new Vec3Array;
         v->resize( 8 );
-        (*v)[0] = Vec3( bb._min.x(), bb._min.y(), bb._min.z() );
-        (*v)[1] = Vec3( bb._max.x(), bb._min.y(), bb._min.z() );
-        (*v)[2] = Vec3( bb._max.x(), bb._min.y(), bb._max.z() );
-        (*v)[3] = Vec3( bb._min.x(), bb._min.y(), bb._max.z() );
-        (*v)[4] = Vec3( bb._max.x(), bb._max.y(), bb._min.z() );
-        (*v)[5] = Vec3( bb._min.x(), bb._max.y(), bb._min.z() );
-        (*v)[6] = Vec3( bb._min.x(), bb._max.y(), bb._max.z() );
-        (*v)[7] = Vec3( bb._max.x(), bb._max.y(), bb._max.z() );
+
+        // Having (0,0,0) as vertices for the case of the invalid query geometry
+        // still isn't quite the right thing. But the query geometry is public
+        // accessible and therefore a user might expect eight vertices, so
+        // it seems safer to keep eight vertices in the geometry.
+
+        if (bbValid)
+        {
+            (*v)[0] = Vec3( bb._min.x(), bb._min.y(), bb._min.z() );
+            (*v)[1] = Vec3( bb._max.x(), bb._min.y(), bb._min.z() );
+            (*v)[2] = Vec3( bb._max.x(), bb._min.y(), bb._max.z() );
+            (*v)[3] = Vec3( bb._min.x(), bb._min.y(), bb._max.z() );
+            (*v)[4] = Vec3( bb._max.x(), bb._max.y(), bb._min.z() );
+            (*v)[5] = Vec3( bb._min.x(), bb._max.y(), bb._min.z() );
+            (*v)[6] = Vec3( bb._min.x(), bb._max.y(), bb._max.z() );
+            (*v)[7] = Vec3( bb._max.x(), bb._max.y(), bb._max.z() );
+        }
 
         Geometry* geom = static_cast< Geometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
         geom->setVertexArray( v.get() );
@@ -593,6 +638,12 @@ void OcclusionQueryNode::setQueriesEnabled( bool enable )
     _enabled = enable;
 }
 
+void OcclusionQueryNode::resetQueries()
+{
+   OpenThreads::ScopedLock<OpenThreads::Mutex> lock( _frameCountMutex );
+   _frameCountMap.clear();
+}
+
 // Should only be called outside of cull/draw. No thread issues.
 void OcclusionQueryNode::setDebugDisplay( bool debug )
 {
@@ -718,13 +769,10 @@ void OcclusionQueryNode::createSupportNodes()
 
 void OcclusionQueryNode::releaseGLObjects( State* state ) const
 {
-    if(_queryGeode->getDrawable( 0 ) != NULL)
-    {
-        // Query object discard and deletion is handled by QueryGeometry support class.
-        OcclusionQueryNode* nonConstThis = const_cast< OcclusionQueryNode* >( this );
-        QueryGeometry* qg = static_cast< QueryGeometry* >( nonConstThis->_queryGeode->getDrawable( 0 ) );
-        qg->releaseGLObjects( state );
-    }
+    if (_queryGeode.valid()) _queryGeode->releaseGLObjects(state);
+    if (_debugGeode.valid()) _debugGeode->releaseGLObjects(state);
+
+    osg::Group::releaseGLObjects(state);
 }
 
 void OcclusionQueryNode::flushDeletedQueryObjects( unsigned int contextID, double currentTime, double& availableTime )
