@@ -32,6 +32,7 @@
 #include <osg/ref_ptr>
 #include <osg/Shader>
 #include <osg/GLExtensions>
+#include <osg/ContextData>
 
 namespace osg
 {
@@ -88,10 +89,34 @@ struct NoneOf
     const char* _str;
 };
 
+// Replaces all occurrences of "from" with the contents of "to"
+// It does only one pass, i.e. new matches created in a position before the current match are not replaced
+void replaceAll(std::string& str, const std::string& from, const std::string& to)
+{
+    std::string::size_type pos = 0;
+    while ((pos = str.find(from, pos)) != std::string::npos)
+    {
+        str.replace(pos, from.length(), to);
+
+        pos += to.length();
+    }
+}
+
 }
 
 using namespace osg;
 
+class GLShaderManager : public GLObjectManager
+{
+public:
+    GLShaderManager(unsigned int contextID) : GLObjectManager("GLShaderManager",contextID) {}
+
+    virtual void deleteGLObject(GLuint globj)
+    {
+        const GLExtensions* extensions = GLExtensions::Get(_contextID,true);
+        if (extensions->isGlslSupported) extensions->glDeleteShader( globj );
+    }
+};
 
 ///////////////////////////////////////////////////////////////////////////////////
 //
@@ -205,64 +230,6 @@ ShaderBinary* ShaderBinary::readShaderBinaryFile(const std::string& fileName)
     return shaderBinary.release();
 }
 
-///////////////////////////////////////////////////////////////////////////
-// static cache of glShaders flagged for deletion, which will actually
-// be deleted in the correct GL context.
-
-typedef std::list<GLuint> GlShaderHandleList;
-typedef osg::buffered_object<GlShaderHandleList> DeletedGlShaderCache;
-
-static OpenThreads::Mutex    s_mutex_deletedGlShaderCache;
-static DeletedGlShaderCache  s_deletedGlShaderCache;
-
-void Shader::deleteGlShader(unsigned int contextID, GLuint shader)
-{
-    if( shader )
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedGlShaderCache);
-
-        // add glShader to the cache for the appropriate context.
-        s_deletedGlShaderCache[contextID].push_back(shader);
-    }
-}
-
-void Shader::flushDeletedGlShaders(unsigned int contextID,double /*currentTime*/, double& availableTime)
-{
-    // if no time available don't try to flush objects.
-    if (availableTime<=0.0) return;
-
-    const GLExtensions* extensions = GLExtensions::Get(contextID,true);
-    if( ! extensions->isGlslSupported ) return;
-
-    const osg::Timer& timer = *osg::Timer::instance();
-    osg::Timer_t start_tick = timer.tick();
-    double elapsedTime = 0.0;
-
-    {
-        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedGlShaderCache);
-
-        GlShaderHandleList& pList = s_deletedGlShaderCache[contextID];
-        for(GlShaderHandleList::iterator titr=pList.begin();
-            titr!=pList.end() && elapsedTime<availableTime;
-            )
-        {
-            extensions->glDeleteShader( *titr );
-            titr = pList.erase( titr );
-            elapsedTime = timer.delta_s(start_tick,timer.tick());
-        }
-    }
-
-    availableTime -= elapsedTime;
-}
-
-void Shader::discardDeletedGlShaders(unsigned int contextID)
-{
-    OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedGlShaderCache);
-
-    GlShaderHandleList& pList = s_deletedGlShaderCache[contextID];
-    pList.clear();
-}
-
 ///////////////////////////////////////////////////////////////////////////
 // osg::Shader
 ///////////////////////////////////////////////////////////////////////////
@@ -347,7 +314,7 @@ void Shader::setShaderSource( const std::string& sourceText )
     dirtyShader();
 }
 
-
+#ifdef OSG_USE_DEPRECATED_API
 Shader* Shader::readShaderFile( Type type, const std::string& fileName )
 {
     ref_ptr<Shader> shader = new Shader(type);
@@ -381,7 +348,7 @@ bool Shader::loadShaderSourceFromFile( const std::string& fileName )
     delete [] text;
     return true;
 }
-
+#endif
 
 const char* Shader::getTypename() const
 {
@@ -516,6 +483,7 @@ Shader::PerContextShader* Shader::getPCS(osg::State& state) const
 
 bool Shader::addProgramRef( Program* program )
 {
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lk(_programSetMutex);
     ProgramSet::iterator itr = _programSet.find(program);
     if( itr != _programSet.end() ) return false;
 
@@ -525,6 +493,7 @@ bool Shader::addProgramRef( Program* program )
 
 bool Shader::removeProgramRef( Program* program )
 {
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lk(_programSetMutex);
     ProgramSet::iterator itr = _programSet.find(program);
     if( itr == _programSet.end() ) return false;
 
@@ -540,6 +509,7 @@ void Shader::dirtyShader()
         if( _pcsList[cxt].valid() ) _pcsList[cxt]->requestCompile();
     }
 
+    OpenThreads::ScopedLock<OpenThreads::Mutex> lk(_programSetMutex);
     // Also mark Programs that depend on us as needing relink.
     for( ProgramSet::iterator itr = _programSet.begin();
         itr != _programSet.end(); ++itr )
@@ -567,7 +537,7 @@ Shader::PerContextShader::PerContextShader(const Shader* shader, unsigned int co
 
 Shader::PerContextShader::~PerContextShader()
 {
-    Shader::deleteGlShader( _contextID, _glShaderHandle );
+    osg::get<GLShaderManager>(_contextID)->scheduleGLObjectForDeletion(_glShaderHandle);
 }
 
 
@@ -613,7 +583,7 @@ void Shader::PerContextShader::compileShader(osg::State& state)
     if( ! _needsCompile ) return;
     _needsCompile = false;
 
-#if defined(OSG_GLES2_AVAILABLE)
+#if defined(OSG_GLES2_AVAILABLE) || defined(OSG_GLES3_AVAILABLE)
     if (_shader->getShaderBinary())
     {
         GLint numFormats = 0;
@@ -664,30 +634,32 @@ void Shader::PerContextShader::compileShader(osg::State& state)
 #endif
 
     std::string source = _shader->getShaderSource();
-    if (_shader->getType()==osg::Shader::VERTEX && (state.getUseVertexAttributeAliasing() || state.getUseModelViewAndProjectionUniforms()))
+    // if (_shader->getType()==osg::Shader::VERTEX && (state.getUseVertexAttributeAliasing() || state.getUseModelViewAndProjectionUniforms()))
     {
         state.convertVertexShaderSourceToOsgBuiltIns(source);
     }
 
 
-    if (osg::getNotifyLevel()>=osg::INFO)
-    {
-        std::string sourceWithLineNumbers = insertLineNumbers(source);
-        OSG_INFO << "\nCompiling " << _shader->getTypename()
-                 << " source:\n" << sourceWithLineNumbers << std::endl;
-    }
-
     GLint compiled = GL_FALSE;
 
-    // OSG_NOTICE<<"Compiling PerContextShader "<<this<<" ShaderDefine="<<getDefineString()<<std::endl;
+    // OSG_NOTICE<<"Compiling PerContextShader "<<this<<" DefineString="<<getDefineString()<<std::endl;
 
     if (_defineStr.empty())
     {
         const GLchar* sourceText = reinterpret_cast<const GLchar*>(source.c_str());
         _extensions->glShaderSource( _glShaderHandle, 1, &sourceText, NULL );
+
+        if (osg::getNotifyLevel()>=osg::INFO)
+        {
+            std::string sourceWithLineNumbers = insertLineNumbers(source);
+            OSG_INFO << "\nCompiling A :" << _shader->getTypename()
+                    << " source:\n" << sourceWithLineNumbers << std::endl;
+        }
     }
     else
     {
+        // Convert all windows line endings to \n
+        replaceAll(source, "\r\n", " \n");
 
         std::string versionLine;
         unsigned int lineNum = 0;
@@ -721,21 +693,40 @@ void Shader::PerContextShader::compileShader(osg::State& state)
 
         if (!versionLine.empty())
         {
-            // OSG_NOTICE<<"Shader::PerContextShader::compileShader() : Found #version,  lineNum = "<<lineNum<<" ["<<versionLine<<"] new source = ["<<source<<"]"<<std::endl;
             const GLchar* sourceText[3];
-            //OSG_NOTICE<<"glShaderSource() ["<<versionLine<<"] "<<std::endl<<"["<<_defineStr<<"], ["<<sourceText<<"]"<<std::endl;
             sourceText[0] = reinterpret_cast<const GLchar*>(versionLine.c_str());
             sourceText[1] = reinterpret_cast<const GLchar*>(_defineStr.c_str());
             sourceText[2] = reinterpret_cast<const GLchar*>(source.c_str());
             _extensions->glShaderSource( _glShaderHandle, 3, sourceText, NULL );
+
+            if (osg::getNotifyLevel()>=osg::INFO)
+            {
+                std::string sourceWithLineNumbers = insertLineNumbers(versionLine+_defineStr+source);
+                OSG_INFO << "\nCompiling B: " << _shader->getTypename()
+                        << " source:\n" << sourceWithLineNumbers << std::endl;
+            }
+
+            // OSG_NOTICE<<"   Version Line : ["<<std::endl<<versionLine<<"]"<<std::endl;
+            // OSG_NOTICE<<"   DefineStr : ["<<std::endl<<_defineStr<<"]"<<std::endl;
+            // OSG_NOTICE<<"   Source : ["<<std::endl<<source<<"]"<<std::endl;
         }
         else
         {
             const GLchar* sourceText[2];
-            //OSG_NOTICE<<"glShaderSource() ["<<_defineStr<<"], ["<<sourceText<<"]"<<std::endl;
             sourceText[0] = reinterpret_cast<const GLchar*>(_defineStr.c_str());
             sourceText[1] = reinterpret_cast<const GLchar*>(source.c_str());
             _extensions->glShaderSource( _glShaderHandle, 2, sourceText, NULL );
+
+
+            if (osg::getNotifyLevel()>=osg::INFO)
+            {
+                std::string sourceWithLineNumbers = insertLineNumbers(_defineStr+source);
+                OSG_INFO << "\nCompiling C: " << _shader->getTypename()
+                        << " source:\n" << sourceWithLineNumbers << std::endl;
+            }
+
+            // OSG_NOTICE<<"   DefineStr : ["<<std::endl<<_defineStr<<"]"<<std::endl;
+            // OSG_NOTICE<<"   Source : ["<<std::endl<<source<<"]"<<std::endl;
         }
     }
     _extensions->glCompileShader( _glShaderHandle );
@@ -762,6 +753,8 @@ void Shader::PerContextShader::compileShader(osg::State& state)
             OSG_INFO << _shader->getTypename() << " Shader \""
                 << _shader->getName() << "\" infolog:\n" << infoLog << std::endl;
         }
+
+        _extensions->debugObjectLabel(GL_SHADER, _glShaderHandle, _shader->getName());
     }
 
 }
