@@ -288,6 +288,9 @@ Registry::Registry()
     addFileExtensionAlias("terrain", "osgterrain");
     addFileExtensionAlias("view",  "osgviewer");
 
+    //addFileExtensionAlias("vsga", "vsg");
+    //addFileExtensionAlias("vsgb", "vsg");
+
     addFileExtensionAlias("sgi",  "rgb");
     addFileExtensionAlias("rgba", "rgb");
     addFileExtensionAlias("int",  "rgb");
@@ -307,6 +310,13 @@ Registry::Registry()
     addFileExtensionAlias("vert", "glsl");
     addFileExtensionAlias("frag", "glsl");
     addFileExtensionAlias("geom", "glsl");
+    addFileExtensionAlias("tctrl", "glsl");
+    addFileExtensionAlias("teval", "glsl");
+    addFileExtensionAlias("compute", "glsl");
+    addFileExtensionAlias("vs", "glsl");
+    addFileExtensionAlias("fs", "glsl");
+    addFileExtensionAlias("cs", "glsl");
+    addFileExtensionAlias("gs", "glsl");
 
     addFileExtensionAlias("js", "V8");
 
@@ -444,6 +454,11 @@ Registry::Registry()
     addFileExtensionAlias("modified", "revisions");
 
 
+    // STEP/IGES file mappings
+    addFileExtensionAlias("stp",  "opencascade");
+    addFileExtensionAlias("step", "opencascade");
+    addFileExtensionAlias("igs",  "opencascade");
+    addFileExtensionAlias("iges", "opencascade");
 
 
     // add built-in mime-type extension mappings
@@ -570,13 +585,19 @@ void Registry::readCommandLine(osg::ArgumentParser& arguments)
     std::string value;
     while(arguments.read("-l",value))
     {
-        loadLibrary(value);
+        if (loadLibrary(value)==NOT_LOADED)
+        {
+            OSG_NOTICE<<"Unable to load library : "<<value<<std::endl;
+        }
     }
 
     while(arguments.read("-e",value))
     {
         std::string libName = createLibraryNameForExtension(value);
-        loadLibrary(libName);
+        if (loadLibrary(libName)==NOT_LOADED)
+        {
+            OSG_NOTICE<<"Unable to load library : "<<libName<<std::endl;
+        }
     }
 
     while(arguments.read("-O",value))
@@ -1092,6 +1113,7 @@ std::string Registry::findLibraryFileImplementation(const std::string& filename,
 
     const FilePathList& filepath = Registry::instance()->getLibraryFilePathList();
 
+
     std::string fileFound = findFileInPath(filename, filepath,caseSensitivity);
     if (!fileFound.empty())
         return fileFound;
@@ -1106,7 +1128,7 @@ std::string Registry::findLibraryFileImplementation(const std::string& filename,
     std::string simpleFileName = getSimpleFileName(filename);
     if (simpleFileName!=filename)
     {
-        std::string fileFound = findFileInPath(simpleFileName, filepath,caseSensitivity);
+        fileFound = findFileInPath(simpleFileName, filepath,caseSensitivity);
         if (!fileFound.empty()) return fileFound;
     }
 
@@ -1149,7 +1171,7 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
 
             options->setDatabasePath(archiveName);
 
-            std::auto_ptr<ReadFunctor> rf(readFunctor.cloneType(fileName, options.get()));
+            osg::ref_ptr<ReadFunctor> rf(readFunctor.cloneType(fileName, options.get()));
 
             result = rf->doRead(*archive);
 
@@ -1216,30 +1238,19 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
         }
         else
         {
-            return  ReaderWriter::ReadResult("Warning: Could not find the .curl plugin to read from server.");
+            return  ReaderWriter::ReadResult("Could not find the .curl plugin to read from server.");
         }
     }
 
     if (results.empty())
     {
-        return ReaderWriter::ReadResult("Warning: Could not find plugin to read objects from file \""+readFunctor._filename+"\".");
+        return ReaderWriter::ReadResult("Could not find plugin to read objects from file \""+readFunctor._filename+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_READING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::ReadResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::ReadResult::FILE_NOT_HANDLED): result.message() = "Warning: reading \""+readFunctor._filename+"\" not supported."; break;
-            case(ReaderWriter::ReadResult::FILE_NOT_FOUND): result.message() = "Warning: could not find file \""+readFunctor._filename+"\"."; break;
-            case(ReaderWriter::ReadResult::ERROR_IN_READING_FILE): result.message() = "Warning: Error in reading to \""+readFunctor._filename+"\"."; break;
-            default: break;
-        }
-    }
-
     return result;
 }
 
@@ -1260,9 +1271,9 @@ ReaderWriter::ReadResult Registry::readImplementation(const ReadFunctor& readFun
     if (useObjectCache)
     {
         // search for entry in the object cache.
-        osg::ref_ptr<osg::Object> object = optionsCache ? optionsCache->getRefFromObjectCache(file) : 0;
+        osg::ref_ptr<osg::Object> object = optionsCache ? optionsCache->getRefFromObjectCache(file, options) : 0;
 
-        if (!object && _objectCache.valid()) object = _objectCache->getRefFromObjectCache(file);
+        if (!object && _objectCache.valid()) object = _objectCache->getRefFromObjectCache(file, options);
 
         if (object.valid())
         {
@@ -1274,7 +1285,7 @@ ReaderWriter::ReadResult Registry::readImplementation(const ReadFunctor& readFun
         if (rr.validObject())
         {
             // search AGAIN for entry in the object cache.
-            object = _objectCache->getRefFromObjectCache(file);
+            object = _objectCache->getRefFromObjectCache(file, options);
             if (object.valid())
             {
                 if (readFunctor.isValid(object.get())) return ReaderWriter::ReadResult(object.get(), ReaderWriter::ReadResult::FILE_LOADED_FROM_CACHE);
@@ -1285,8 +1296,8 @@ ReaderWriter::ReadResult Registry::readImplementation(const ReadFunctor& readFun
             }
 
             // update cache with new entry.
-            if (optionsCache) optionsCache->addEntryToObjectCache(file, rr.getObject(), 0.0);
-            else if (_objectCache.valid()) _objectCache->addEntryToObjectCache(file, rr.getObject(), 0.0);
+            if (optionsCache) optionsCache->addEntryToObjectCache(file, rr.getObject(), 0.0, options);
+            else if (_objectCache.valid()) _objectCache->addEntryToObjectCache(file, rr.getObject(), 0.0, options);
         }
         else
         {
@@ -1356,23 +1367,13 @@ ReaderWriter::WriteResult Registry::writeObjectImplementation(const Object& obj,
 
     if (results.empty())
     {
-        return ReaderWriter::WriteResult("Warning: Could not find plugin to write objects to file \""+fileName+"\".");
+        return ReaderWriter::WriteResult("Could not find plugin to write objects to file \""+fileName+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_WRITING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::WriteResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): result.message() = "Warning: Write to \""+fileName+"\" not supported."; break;
-            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): result.message() = "Warning: Error in writing to \""+fileName+"\"."; break;
-            default: break;
-        }
-    }
-
     return result;
 }
 
@@ -1412,23 +1413,13 @@ ReaderWriter::WriteResult Registry::writeImageImplementation(const Image& image,
 
     if (results.empty())
     {
-        return ReaderWriter::WriteResult("Warning: Could not find plugin to write image to file \""+fileName+"\".");
+        return ReaderWriter::WriteResult("Could not find plugin to write image to file \""+fileName+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_WRITING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::WriteResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): result.message() = "Warning: Write to \""+fileName+"\" not supported."; break;
-            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): result.message() = "Warning: Error in writing to \""+fileName+"\"."; break;
-            default: break;
-        }
-    }
-
     return result;
 }
 
@@ -1467,23 +1458,13 @@ ReaderWriter::WriteResult Registry::writeHeightFieldImplementation(const HeightF
 
     if (results.empty())
     {
-        return ReaderWriter::WriteResult("Warning: Could not find plugin to write HeightField to file \""+fileName+"\".");
+        return ReaderWriter::WriteResult("Could not find plugin to write HeightField to file \""+fileName+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_WRITING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::WriteResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): result.message() = "Warning: Write to \""+fileName+"\" not supported."; break;
-            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): result.message() = "Warning: Error in writing to \""+fileName+"\"."; break;
-            default: break;
-        }
-    }
-
     return result;
 }
 
@@ -1536,23 +1517,13 @@ ReaderWriter::WriteResult Registry::writeNodeImplementation(const Node& node,con
 
     if (results.empty())
     {
-        return ReaderWriter::WriteResult("Warning: Could not find plugin to write nodes to file \""+fileName+"\".");
+        return ReaderWriter::WriteResult("Could not find plugin to write nodes to file \""+fileName+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_WRITING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::WriteResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): result.message() = "Warning: Write to \""+fileName+"\" not supported."; break;
-            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): result.message() = "Warning: Error in writing to \""+fileName+"\"."; break;
-            default: break;
-        }
-    }
-
     return result;
 }
 
@@ -1592,22 +1563,13 @@ ReaderWriter::WriteResult Registry::writeShaderImplementation(const Shader& shad
 
     if (results.empty())
     {
-        return ReaderWriter::WriteResult("Warning: Could not find plugin to write shader to file \""+fileName+"\".");
+        return ReaderWriter::WriteResult("Could not find plugin to write shader to file \""+fileName+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_WRITING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::WriteResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): result.message() = "Warning: Write to \""+fileName+"\" not supported."; break;
-            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): result.message() = "Warning: Error in writing to \""+fileName+"\"."; break;
-            default: break;
-        }
-    }
 
     return result;
 }
@@ -1646,39 +1608,29 @@ ReaderWriter::WriteResult Registry::writeScriptImplementation(const Script& imag
 
     if (results.empty())
     {
-        return ReaderWriter::WriteResult("Warning: Could not find plugin to write image to file \""+fileName+"\".");
+        return ReaderWriter::WriteResult("Could not find plugin to write image to file \""+fileName+"\".");
     }
 
     // sort the results so the most relevant (i.e. ERROR_IN_WRITING_FILE is more relevant than FILE_NOT_FOUND) results get placed at the end of the results list.
     std::sort(results.begin(), results.end());
     ReaderWriter::WriteResult result = results.back();
 
-    if (result.message().empty())
-    {
-        switch(result.status())
-        {
-            case(ReaderWriter::WriteResult::FILE_NOT_HANDLED): result.message() = "Warning: Write to \""+fileName+"\" not supported."; break;
-            case(ReaderWriter::WriteResult::ERROR_IN_WRITING_FILE): result.message() = "Warning: Error in writing to \""+fileName+"\"."; break;
-            default: break;
-        }
-    }
-
     return result;
 }
 
-void Registry::addEntryToObjectCache(const std::string& filename, osg::Object* object, double timestamp)
+void Registry::addEntryToObjectCache(const std::string& filename, osg::Object* object, double timestamp, Options *options)
 {
-    if (_objectCache.valid()) _objectCache->addEntryToObjectCache(filename, object, timestamp);
+    if (_objectCache.valid()) _objectCache->addEntryToObjectCache(filename, object, timestamp, options);
 }
 
-osg::Object* Registry::getFromObjectCache(const std::string& filename)
+osg::Object* Registry::getFromObjectCache(const std::string& filename, Options *options)
 {
-    return _objectCache.valid() ? _objectCache->getFromObjectCache(filename) : 0;
+    return _objectCache.valid() ? _objectCache->getFromObjectCache(filename, options) : 0;
 }
 
-osg::ref_ptr<osg::Object> Registry::getRefFromObjectCache(const std::string& filename)
+osg::ref_ptr<osg::Object> Registry::getRefFromObjectCache(const std::string& filename, Options *options)
 {
-    return _objectCache.valid() ? _objectCache->getRefFromObjectCache(filename) : 0;
+    return _objectCache.valid() ? _objectCache->getRefFromObjectCache(filename, options) : 0;
 }
 
 void Registry::updateTimeStampOfObjectsInCacheWithExternalReferences(const osg::FrameStamp& frameStamp)
@@ -1692,9 +1644,9 @@ void Registry::removeExpiredObjectsInCache(const osg::FrameStamp& frameStamp)
     if (_objectCache.valid()) _objectCache->removeExpiredObjectsInCache(expiryTime);
 }
 
-void Registry::removeFromObjectCache(const std::string& filename)
+void Registry::removeFromObjectCache(const std::string& filename, Options *options)
 {
-    if (_objectCache.valid()) _objectCache->removeFromObjectCache(filename);
+    if (_objectCache.valid()) _objectCache->removeFromObjectCache(filename, options);
 }
 
 void Registry::clearObjectCache()
