@@ -21,6 +21,7 @@
 #endif
 
 #include <stdlib.h>
+#include <limits>
 #include <string>
 
 #include <osg/Notify>
@@ -41,7 +42,7 @@
 #include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
 
-#include <osgUtil/TriStripVisitor>
+#include <osgUtil/MeshOptimizers>
 #include <osgUtil/SmoothingVisitor>
 #include <osgUtil/Tessellator>
 
@@ -60,7 +61,7 @@ public:
         supportsOption("noRotation","Do not do the default rotate about X axis");
         supportsOption("noTesselateLargePolygons","Do not do the default tesselation of large polygons");
         supportsOption("noTriStripPolygons","Do not do the default tri stripping of polygons");
-        supportsOption("generateFacetNormals","generate facet normals for verticies without normals");
+        supportsOption("generateFacetNormals","generate facet normals for vertices without normals");
         supportsOption("noReverseFaces","avoid to reverse faces when normals and triangles orientation are reversed");
 
         supportsOption("DIFFUSE=<unit>", "Set texture unit for diffuse texture");
@@ -71,11 +72,24 @@ public:
         supportsOption("BUMP=<unit>", "Set texture unit for bumpmap texture");
         supportsOption("DISPLACEMENT=<unit>", "Set texture unit for displacement texture");
         supportsOption("REFLECTION=<unit>", "Set texture unit for reflection texture");
+        supportsOption("NsIfNotPresent=<value>", "set specular exponent if not present");
 
+        supportsOption("precision=<digits>","Set the floating point precision when writing out files");
+        supportsOption("OutputTextureFiles", "Write out the texture images to file");
     }
 
     virtual const char* className() const { return "Wavefront OBJ Reader"; }
 
+    virtual ReadResult readObject(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const
+    {
+        return readNode(fileName, options);
+    }
+
+    virtual ReadResult readObject(std::istream& fin, const Options* options) const
+    {
+        return readNode(fin, options);
+    }
+
     virtual ReadResult readNode(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const;
 
     virtual ReadResult readNode(std::istream& fin, const Options* options) const;
@@ -89,14 +103,18 @@ public:
             return WriteResult(WriteResult::FILE_NOT_HANDLED);
     }
 
-    virtual WriteResult writeNode(const osg::Node& node,const std::string& fileName,const Options* options =NULL) const
+    virtual WriteResult writeNode(const osg::Node& node,const std::string& fileName,const Options* options=NULL) const
     {
         if (!acceptsExtension(osgDB::getFileExtension(fileName)))
             return WriteResult(WriteResult::FILE_NOT_HANDLED);
 
+        ObjOptionsStruct localOptions = parseOptions(options);
+
         osgDB::ofstream f(fileName.c_str());
+        f.precision(localOptions.precision);
+
         std::string materialFile = osgDB::getNameLessExtension(fileName) + ".mtl";
-        OBJWriterNodeVisitor nv(f, osgDB::getSimpleFileName(materialFile));
+        OBJWriterNodeVisitor nv(f, osgDB::getSimpleFileName(materialFile), localOptions.outputTextureFiles, options);
 
         // we must cast away constness
         (const_cast<osg::Node*>(&node))->accept(nv);
@@ -117,11 +135,14 @@ public:
             return WriteResult(WriteResult::FILE_NOT_HANDLED);
     }
 
-    virtual WriteResult writeNode(const osg::Node& node,std::ostream& fout,const Options* =NULL) const
+    virtual WriteResult writeNode(const osg::Node& node,std::ostream& fout,const Options* options=NULL) const
     {
+        ObjOptionsStruct localOptions = parseOptions(options);
+        fout.precision(localOptions.precision);
+
         // writing to a stream does not support materials
 
-        OBJWriterNodeVisitor nv(fout);
+        OBJWriterNodeVisitor nv(fout, "", localOptions.outputTextureFiles, options);
 
         // we must cast away constness
         (const_cast<osg::Node*>(&node))->accept(nv);
@@ -133,7 +154,8 @@ public:
 
 protected:
 
-     struct ObjOptionsStruct {
+     class ObjOptionsStruct {
+     public:
         bool rotate;
         bool noTesselateLargePolygons;
         bool noTriStripPolygons;
@@ -141,9 +163,26 @@ protected:
         bool fixBlackMaterials;
         bool noReverseFaces;
         // This is the order in which the materials will be assigned to texture maps, unless
-        // otherwise overriden
+        // otherwise overridden
         typedef std::vector< std::pair<int,obj::Material::Map::TextureMapType> > TextureAllocationMap;
         TextureAllocationMap textureUnitAllocation;
+        /// Coordinates precision.
+        int precision;
+        bool outputTextureFiles;
+        int specularExponent;
+
+        ObjOptionsStruct()
+        {
+            rotate = true;
+            noTesselateLargePolygons = false;
+            noTriStripPolygons = false;
+            generateFacetNormals = false;
+            fixBlackMaterials = true;
+            noReverseFaces = false;
+            precision = std::numeric_limits<double>::digits10 + 2;
+            outputTextureFiles = false;
+            specularExponent = -1;
+        }
     };
 
     typedef std::map< std::string, osg::ref_ptr<osg::StateSet> > MaterialToStateSetMap;
@@ -275,7 +314,7 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
 {
     if (localOptions.fixBlackMaterials)
     {
-        // hack to fix Maya exported models that contian all black materials.
+        // hack to fix Maya exported models that contain all black materials.
         int numBlack = 0;
         int numNotBlack = 0;
         obj::Model::MaterialMap::iterator itr;
@@ -333,12 +372,13 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
             osg_material->setDiffuse(osg::Material::FRONT_AND_BACK,material.diffuse);
             osg_material->setEmission(osg::Material::FRONT_AND_BACK,material.emissive);
 
-            if (material.illum == 2) {
+            if (material.illum >= 2) {
                 osg_material->setSpecular(osg::Material::FRONT_AND_BACK,material.specular);
             } else {
                 osg_material->setSpecular(osg::Material::FRONT_AND_BACK, osg::Vec4(0,0,0,1));
             }
-            osg_material->setShininess(osg::Material::FRONT_AND_BACK,(material.Ns/1000.0f)*128.0f ); // note OBJ shiniess is 0..1000.
+            int ns = material.Ns != -1 ? material.Ns : localOptions.specularExponent != -1 ? localOptions.specularExponent : 0;
+            osg_material->setShininess(osg::Material::FRONT_AND_BACK,(ns/1000.0f)*128.0f ); // note OBJ shiniess is 0..1000.
 
             if (material.ambient[3]!=1.0 ||
                 material.diffuse[3]!=1.0 ||
@@ -510,8 +550,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
 
     if (colors)
     {
-        geometry->setColorArray(colors);
-        geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
+        geometry->setColorArray(colors, osg::Array::BIND_PER_VERTEX);
     }
 
     if (numPointElements>0)
@@ -631,12 +670,6 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
             obj::Element& element = *(*itr);
             if (element.dataType==obj::Element::POLYGON)
             {
-
-
-
-
-
-
                 #ifdef USE_DRAWARRAYLENGTHS
                     drawArrayLengths->push_back(element.vertexIndices.size());
                 #else
@@ -778,11 +811,10 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptio
                 tessellator.retessellatePolygons(*geometry);
             }
 
-            // tri strip polygons to improve graphics peformance
+            // tri strip polygons to improve graphics performance
             if (!localOptions.noTriStripPolygons)
             {
-                osgUtil::TriStripVisitor tsv;
-                tsv.stripify(*geometry);
+                osgUtil::optimizeMesh(geometry);
             }
 
             // if no normals present add them.
@@ -820,12 +852,6 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, ObjOptio
 ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::ReaderWriter::Options* options) const
 {
     ObjOptionsStruct localOptions;
-    localOptions.rotate = true;
-    localOptions.noTesselateLargePolygons = false;
-    localOptions.noTriStripPolygons = false;
-    localOptions.generateFacetNormals = false;
-    localOptions.fixBlackMaterials = true;
-    localOptions.noReverseFaces = false;
 
     if (options!=NULL)
     {
@@ -868,6 +894,25 @@ ReaderWriterOBJ::ObjOptionsStruct ReaderWriterOBJ::parseOptions(const osgDB::Rea
             {
                 localOptions.noReverseFaces = true;
             }
+            else if (pre_equals == "OutputTextureFiles")
+            {
+                localOptions.outputTextureFiles = true;
+            }
+            else if (pre_equals == "precision")
+            {
+                int val = std::atoi(post_equals.c_str());
+                if (val <= 0) {
+                    OSG_NOTICE << "Warning: invalid precision value: " << post_equals << std::endl;
+                }
+                else {
+                    localOptions.precision = val;
+                }
+            }
+            else if (pre_equals == "NsIfNotPresent")
+            {
+                int value = atoi(post_equals.c_str());
+                localOptions.specularExponent = value ;
+            }
             else if (post_equals.length()>0)
             {
                 obj::Material::Map::TextureMapType type = obj::Material::Map::UNKNOWN;
