@@ -231,7 +231,6 @@ bool Model::readMTL(std::istream& fin)
     bool usingDissolve = false;
 
     Material* material = 0;// &(materialMap[""]);
-    std::string filename;
 
     while (fin)
     {
@@ -507,7 +506,7 @@ bool Model::readMTL(std::istream& fin)
                     material->maps.push_back(parseTextureMap(strip(line+7),Material::Map::SPECULAR_EXPONENT));
                 }
                 // modelling tools and convertors variously produce bump, map_bump, and map_Bump so parse them all
-                else if (strncasecmp(line,"bump ",5)==0 || strncasecmp(line,"map_bump ",9)==0)
+                else if (strncasecmp(line,"bump ",5)==0)
                 {
                     material->maps.push_back(parseTextureMap(strip(line+5),Material::Map::BUMP));
                 }
@@ -797,7 +796,14 @@ bool Model::readOBJ(std::istream& fin, const osgDB::ReaderWriter::Options* optio
             {
                 int smoothingGroup=0;
                 if (strncmp(line+2,"off",3)==0) smoothingGroup = 0;
-                else sscanf(line+2,"%d",&smoothingGroup);
+                else
+                {
+                    int result = sscanf(line+2,"%d",&smoothingGroup);
+                    if (result!=1)
+                    {
+                        OSG_NOTICE <<"*** error reading smoothing group ***"<<std::endl;
+                    }
+                }
 
                 if (currentElementState.smoothingGroup != smoothingGroup)
                 {
