@@ -32,7 +32,7 @@
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
 
-#include <osgUtil/TriStripVisitor>
+#include <osgUtil/MeshOptimizers>
 #include <osgUtil/SmoothingVisitor>
 #include <osg/TriangleFunctor>
 
@@ -45,6 +45,7 @@
 
 #include <string.h>
 #include <memory>
+#include <iomanip>
 
 struct STLOptionsStruct {
     bool smooth;
@@ -109,11 +110,16 @@ public:
         return "STL Reader";
     }
 
+    virtual ReadResult readObject(const std::string& fileName, const osgDB::ReaderWriter::Options* options) const
+    {
+        return readNode(fileName, options); 
+    }
+
     virtual ReadResult readNode(const std::string& fileName, const osgDB::ReaderWriter::Options*) const;
     virtual WriteResult writeNode(const osg::Node& node, const std::string& fileName, const Options* = NULL) const;
 
 private:
-    class ReaderObject
+    class ReaderObject : public osg::Referenced
     {
     public:
         ReaderObject(bool noTriStripPolygons, bool generateNormals = true):
@@ -182,8 +188,7 @@ private:
             geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES, 0, _numFacets * 3));
 
             if(!_noTriStripPolygons) {
-                osgUtil::TriStripVisitor tristripper;
-                tristripper.stripify(*geom);
+                osgUtil::optimizeMesh(geom.get());
             }
 
             return geom;
@@ -250,7 +255,8 @@ private:
     public:
         CreateStlVisitor(std::string const & fout, const osgDB::ReaderWriter::Options* options = 0):
             osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ACTIVE_CHILDREN),
-            counter(0)
+            counter(0),
+            m_f(0)
         {
             m_localOptions = parseOptions(options);
             if (m_localOptions.separateFiles)
@@ -293,6 +299,7 @@ private:
             else
                 *m_f << "solid " << node.getName() << std::endl;
 
+            *m_f << std::fixed << std::setprecision(7);
             for (unsigned int i = 0; i < node.getNumDrawables(); ++i)
             {
                 osg::TriangleFunctor<PushPoints> tf;
@@ -346,7 +353,7 @@ private:
             osg::Matrix m_mat;
             bool m_dontSaveNormals;
 
-            inline void operator () (const osg::Vec3& _v1, const osg::Vec3& _v2, const osg::Vec3& _v3, bool treatVertexDataAsTemporary)
+            inline void operator () (const osg::Vec3& _v1, const osg::Vec3& _v2, const osg::Vec3& _v3)
             {
                 osg::Vec3 v1 = _v1 * m_mat;
                 osg::Vec3 v2 = _v2 * m_mat;
@@ -399,19 +406,21 @@ const float StlColorDepth = float(StlColorSize); // 2^5 - 1
 // Magics files have a header with a "COLOR=" field giving the color of the whole model
 bool fileComesFromMagics(FILE *fp, osg::Vec4& magicsColor)
 {
-    char header[80];
+    std::string header(80, 0);
     const float magicsColorDepth = 255.f;
 
     ::rewind(fp);
 
-    size_t bytes_read = fread((void*) &header, sizeof(header), 1, fp);
-    if (bytes_read!=sizeof(header)) return false;
+    if (fread((void*) &(*header.begin()), header.size(), 1, fp) != 1)
+        return false;
 
-    ::fseek(fp, sizeof_StlHeader, SEEK_SET);
+    if (::fseek(fp, sizeof_StlHeader, SEEK_SET)!=0)
+    {
+        return false;
+    }
 
     std::string magicsColorPattern ("COLOR=");
-    std::string headerStr = std::string(header);
-    if(size_t colorFieldPos = headerStr.find(magicsColorPattern) != std::string::npos)
+    if(size_t colorFieldPos = header.find(magicsColorPattern) != std::string::npos)
     {
         int colorIndex = colorFieldPos + magicsColorPattern.size() - 1;
         float r = (uint8_t)header[colorIndex] / magicsColorDepth;
@@ -437,7 +446,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterSTL::readNode(const std::string& fil
 
     if (sizeof(unsigned int) != 4)
     {
-        OSG_NOTICE<<"Waring: STL reading not supported as unsigned int is not 4 bytes on this system."<<std::endl;
+        OSG_NOTICE<<"Warning: STL reading not supported as unsigned int is not 4 bytes on this system."<<std::endl;
         return ReadResult::ERROR_IN_READING_FILE;
     }
 
@@ -475,11 +484,21 @@ osgDB::ReaderWriter::ReadResult ReaderWriterSTL::readNode(const std::string& fil
         return ReadResult::ERROR_IN_READING_FILE;
     }
 
-    if (stb.st_size == expectLen)
+    std::string header_text(header.text, sizeof(header.text));
+
+    if (header_text.find("solid") == std::string::npos || stb.st_size == expectLen)
     {
+        if(stb.st_size < expectLen)
+        {
+            unsigned int facets = (stb.st_size - sizeof_StlHeader) / sizeof_StlFacet;
+            OSG_WARN << "Warning: [[stl]] Incomplete file. "
+                     << "Attempting to read " << facets << " out of " << expectFacets << " facets expected."
+                     << std::endl;
+            expectFacets = facets;
+        }
         isBinary = true;
     }
-    else if (strstr(header.text, "solid") != 0)
+    else if (header_text.find("solid") != std::string::npos)
     {
         isBinary = false;
     }
@@ -493,7 +512,13 @@ osgDB::ReaderWriter::ReadResult ReaderWriterSTL::readNode(const std::string& fil
     if (!isBinary)
     {
         fclose(fp);
+
         fp = osgDB::fopen(fileName.c_str(), "r");
+
+        if (!fp)
+        {
+            return ReadResult::FILE_NOT_FOUND;
+        }
     }
 
     osg::ref_ptr<osg::Group> group = new osg::Group;
@@ -508,7 +533,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterSTL::readNode(const std::string& fil
     else
         readerObject = new AsciiReaderObject(localOptions.noTriStripPolygons);
 
-    std::auto_ptr<ReaderObject> readerPtr(readerObject);
+    osg::ref_ptr<ReaderObject> readerPtr(readerObject);
 
     while (1)
     {
@@ -568,9 +593,11 @@ ReaderWriterSTL::ReaderObject::ReadResult ReaderWriterSTL::AsciiReaderObject::re
 
     while (fgets(buf, sizeof(buf), fp))
     {
-        // strip '\n' or '\r\n' and trailing whitespace
-        unsigned int len = strlen(buf) - 1;
+        unsigned int len = strlen(buf);
+        if (len==0) continue;
 
+        // strip '\n' or '\r\n' and trailing whitespace
+        --len;
         while (len && (buf[len] == '\n' || buf[len] == '\r' || isspace(buf[len])))
         {
             buf[len--] = '\0';
@@ -672,7 +699,10 @@ ReaderWriterSTL::ReaderObject::ReadResult ReaderWriterSTL::BinaryReaderObject::r
     bool comesFromMagics = fileComesFromMagics(fp, magicsHeaderColor);
 
     // seek to beginning of facets
-    ::fseek(fp, sizeof_StlHeader, SEEK_SET);
+    if (::fseek(fp, sizeof_StlHeader, SEEK_SET)!=0)
+    {
+        return ReadError;
+    }
 
     StlFacet facet;
     for (unsigned int i = 0; i < _expectNumFacets; ++i)
@@ -718,7 +748,7 @@ ReaderWriterSTL::ReaderObject::ReadResult ReaderWriterSTL::BinaryReaderObject::r
          *
          * The magics files may use whether per-face or per-object colors
          * for a given face, according to the value of the last bit (0 = per-face, 1 = per-object)
-         * Moreover, magics uses RGB instead of BGR (as the other softwares)
+         * Moreover, magics uses RGB instead of BGR (as the other software)
          */
         if (!_color.valid())
         {
