@@ -222,7 +222,7 @@ toff_t libtiffOStreamSeekProc(thandle_t fd, toff_t off, int i)
         // position required past the end of the stream so we need to insert extra characters to
         // ensure the stream is big enough to encompass the new the position.
         fout->seekp(0, std::ios::end);
-        for(toff_t i=stream_end; i<pos_required; ++i)
+        for(toff_t is=stream_end; is<pos_required; ++is)
         {
             fout->put(char(0));
         }
@@ -269,7 +269,7 @@ simage_tiff_error(char * buffer, int buflen)
 
 /// Generates a std::string from a printf format string and a va_list.
 /// Took & adapted from the man page of printf.
-///\todo Externalize this function to make is useable for all OSG?
+///\todo Externalize this function to make is usable for all OSG?
 std::string doFormat(const char* fmt, va_list ap) {
     static const int MSG_BUFSIZE = 256;            // Initial size of the buffer used for formatting
     static const int MAX_BUFSIZE = 256*1024;    // Maximum size of the buffer used for formatting
@@ -672,13 +672,18 @@ simage_tiff_load(std::istream& fin,
 
         case pack(PHOTOMETRIC_PALETTE, PLANARCONFIG_CONTIG):
         case pack(PHOTOMETRIC_PALETTE, PLANARCONFIG_SEPARATE):
+
             if (TIFFGetField(in, TIFFTAG_COLORMAP, &red, &green, &blue) != 1)
+            {
                 tifferror = ERR_READ;
+                break;
+            }
+
             /* */
             /* Convert 16-bit colormap to 8-bit (unless it looks */
             /* like an old-style 8-bit colormap). */
             /* */
-            if (!tifferror && checkcmap(1<<bitspersample, red, green, blue) == 16)
+            if (!tifferror && bitspersample!=32 && checkcmap(1<<bitspersample, red, green, blue) == 16)
             {
                 int i;
                 for (i = (1<<bitspersample)-1; i >= 0; i--)
@@ -909,7 +914,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             }
 
             uint32 rowsperstrip = 0;
-            
+
             switch(img.getDataType()){
                 case GL_FLOAT:
                     TIFFSetField(image, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_IEEEFP);
@@ -937,7 +942,7 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             TIFFSetField(image, TIFFTAG_COMPRESSION, compressionType);
             TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
             TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
-            
+
             if (rowsperstrip==0) rowsperstrip = TIFFDefaultStripSize(image, 0);
             TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, rowsperstrip);
 
