@@ -122,13 +122,13 @@ bool TXPArchive::openFile(const std::string& archiveName)
         return false;
     }
 
-    const trpgHeader *header = GetHeader();
-    if (header)
+    const trpgHeader* trpgheader = GetHeader();
+    if (trpgheader)
     {
-        header->GetNumLods(_numLODs);
-        header->GetExtents(_swExtents,_neExtents);
-        header->GetVersion(_majorVersion, _minorVersion);
-    _isMaster = header->GetIsMaster();
+        trpgheader->GetNumLods(_numLODs);
+        trpgheader->GetExtents(_swExtents,_neExtents);
+        trpgheader->GetVersion(_majorVersion, _minorVersion);
+        _isMaster = trpgheader->GetIsMaster();
     }
 
     int numTextures;
@@ -152,10 +152,14 @@ bool TXPArchive::loadMaterial(int ix)
     if (GetStatesMapEntry(ix).get())
     return true;
 
-    osg::StateSet* osg_state_set = new osg::StateSet;
+    osg::ref_ptr<osg::StateSet> osg_state_set = new osg::StateSet;
 
-    const trpgMaterial *mat;
-    mat = materialTable.GetMaterialRef(0,i);
+    const trpgMaterial* mat = materialTable.GetMaterialRef(0,i);
+    if (!mat)
+    {
+        OSG_WARN<<"TXPArchive::loadMaterial("<<ix<<") failed."<<std::endl;
+        return false;
+    }
 
     // Set texture
     int numMatTex;
@@ -168,7 +172,7 @@ bool TXPArchive::loadMaterial(int ix)
     //         materials per polygon.
     if( numMatTex )
     {
-        osg::Material *osg_material     = new osg::Material;
+        osg::ref_ptr<osg::Material> osg_material     = new osg::Material;
 
         float64 alpha;
         mat->GetAlpha(alpha);
@@ -193,7 +197,7 @@ bool TXPArchive::loadMaterial(int ix)
         osg_material->setShininess(osg::Material::FRONT_AND_BACK , (float)shinines);
 
         osg_material->setAlpha(osg::Material::FRONT_AND_BACK ,(float)alpha);
-        osg_state_set->setAttributeAndModes(osg_material, osg::StateAttribute::ON);
+        osg_state_set->setAttributeAndModes(osg_material.get(), osg::StateAttribute::ON);
 
         SetUserDataToMaterialAttributes(*osg_state_set, *mat);
 
@@ -221,7 +225,7 @@ bool TXPArchive::loadMaterial(int ix)
             mat->GetTexture(ntex,texId,texEnv);
 
             // Set up texture environment
-            osg::TexEnv       *osg_texenv       = new osg::TexEnv();
+            osg::ref_ptr<osg::TexEnv> osg_texenv       = new osg::TexEnv();
             int32 te_mode;
             texEnv.GetEnvMode(te_mode);
             switch( te_mode )
@@ -240,7 +244,7 @@ bool TXPArchive::loadMaterial(int ix)
                 break;
             }
 
-            osg_state_set->setTextureAttribute(ntex,osg_texenv);
+            osg_state_set->setTextureAttribute(ntex,osg_texenv.get());
 
             int wrap_s, wrap_t;
             texEnv.GetWrap(wrap_s, wrap_t);
@@ -319,7 +323,7 @@ bool TXPArchive::loadMaterial(int ix)
         // Culling mode in txp means opposite from osg i.e. Front-> show front face
         if( cullMode != trpgMaterial::FrontAndBack)
         {
-            osg::CullFace* cull_face = new osg::CullFace;
+            osg::ref_ptr<osg::CullFace> cull_face = new osg::CullFace;
             switch (cullMode)
             {
             case trpgMaterial::Front:
@@ -329,7 +333,7 @@ bool TXPArchive::loadMaterial(int ix)
                 cull_face->setMode(osg::CullFace::FRONT);
                 break;
             }
-            osg_state_set->setAttributeAndModes(cull_face, osg::StateAttribute::ON);
+            osg_state_set->setAttributeAndModes(cull_face.get(), osg::StateAttribute::ON);
         }
     }
     SetStatesMap(i,osg_state_set);
@@ -589,10 +593,10 @@ void trim(std::string& str)
 }
 bool TXPArchive::loadTextStyles()
 {
-    const trpgTextStyleTable *textStyleTable = GetTextStyleTable();
-    if ( !textStyleTable )
+    const trpgTextStyleTable* textStyleTablePtr = GetTextStyleTable();
+    if ( !textStyleTablePtr )
     return false;
-    if ( textStyleTable->GetNumStyle() < 1 )
+    if ( textStyleTablePtr->GetNumStyle() < 1 )
     return true;
 
     // try fontmap.txt
@@ -629,7 +633,7 @@ bool TXPArchive::loadTextStyles()
     OSG_NOTICE << "txp:: All fonts defaulted to arial.ttf" << std::endl;
     }
 
-    const trpgTextStyleTable::StyleMapType *smap = textStyleTable->getStyleMap();
+    const trpgTextStyleTable::StyleMapType *smap = textStyleTablePtr->getStyleMap();
     trpgTextStyleTable::StyleMapType::const_iterator itr = smap->begin();
     for (  ; itr != smap->end(); itr++)
     {
@@ -668,13 +672,13 @@ bool TXPArchive::loadTextStyles()
     return true;
 }
 
-void TXPArchive::addLightAttribute(osgSim::LightPointNode* lpn, osg::StateSet* fallback, const osg::Vec3& att,int handle)
+void TXPArchive::addLightAttribute(osgSim::LightPointNode* lpn, osg::StateSet* fallback, const osg::Vec3& att,int light_handle)
 {
     DeferredLightAttribute la;
     la.lightPoint = lpn;
     la.fallback = fallback;
     la.attitude = att;
-    _lights[handle] = la;
+    _lights[light_handle] = la;
 }
 
 bool TXPArchive::getTileInfo(const TileLocationInfo& loc, TileInfo& info)
@@ -797,13 +801,13 @@ public:
         offset[0] -= bbox._min[0];
         offset[1] -= bbox._min[1];
 
-        trpg2dPoint offsetXY, tileID(_tileInfo.x,_tileInfo.y);
+        trpg2dPoint offsetXY, local_tileID(_tileInfo.x,_tileInfo.y);
         int divider = (0x01 << _tileInfo.lod);
         // calculate which tile model is located in
         tileExtents.x /= divider;
         tileExtents.y /= divider;
-        offset[0] -= tileID.x*tileExtents.x;
-        offset[1] -= tileID.y*tileExtents.y;
+        offset[0] -= local_tileID.x*tileExtents.x;
+        offset[1] -= local_tileID.y*tileExtents.y;
 
         osg::Matrix mat(xform.getMatrix());
         mat.setTrans(offset);
@@ -868,11 +872,11 @@ osg::Group* TXPArchive::getTileContent(
 
     if(childRef)
     {
-        TileLocationInfo loc;
-        childRef->GetTileLoc(loc.x, loc.y, loc.lod);
-        childRef->GetTileZValue(loc.zmin, loc.zmax);
-        childRef->GetTileAddress(loc.addr);
-        childInfoList.push_back(loc);
+        TileLocationInfo child_loc;
+        childRef->GetTileLoc(child_loc.x, child_loc.y, child_loc.lod);
+        childRef->GetTileZValue(child_loc.zmin, child_loc.zmax);
+        childRef->GetTileAddress(child_loc.addr);
+        childInfoList.push_back(child_loc);
 
     }
     }
