@@ -168,8 +168,8 @@ VDSMCameraCullCallback::VDSMCameraCullCallback(ViewDependentShadowMap* vdsm, osg
 
 void VDSMCameraCullCallback::operator()(osg::Node* node, osg::NodeVisitor* nv)
 {
-    osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(nv);
-    osg::Camera* camera = dynamic_cast<osg::Camera*>(node);
+    osgUtil::CullVisitor* cv = nv->asCullVisitor();
+    osg::Camera* camera = node->asCamera();
     OSG_INFO<<"VDSMCameraCullCallback::operator()(osg::Node* "<<camera<<", osg::NodeVisitor* "<<cv<<")"<<std::endl;
 
 #if 1
@@ -274,20 +274,14 @@ public:
         popCurrentMask();
     }
 
-    void apply(osg::Geode& node)
+    void apply(osg::Drawable& drawable)
     {
-        if (isCulled(node)) return;
+        if (isCulled(drawable)) return;
 
         // push the culling mode.
         pushCurrentMask();
 
-        for(unsigned int i=0; i<node.getNumDrawables();++i)
-        {
-            if (node.getDrawable(i))
-            {
-                updateBound(node.getDrawable(i)->getBoundingBox());
-            }
-        }
+        updateBound(drawable.getBoundingBox());
 
         // pop the culling mode.
         popCurrentMask();
@@ -1947,7 +1941,6 @@ struct RenderLeafBounds
 
     osg::Matrixd        light_mvp;
     osg::RefMatrix*     previous_modelview;
-    unsigned int        numLeaves;
 
     double clip_min_x, clip_max_x;
     double clip_min_y, clip_max_y;
