@@ -30,8 +30,8 @@ CompositeViewer::CompositeViewer()
     constructorInit();
 }
 
-CompositeViewer::CompositeViewer(const CompositeViewer& cv,const osg::CopyOp& /*copyop*/):
-    osg::Object(true),
+CompositeViewer::CompositeViewer(const CompositeViewer& cv,const osg::CopyOp& copyop):
+    osg::Object(cv, copyop),
     ViewerBase(cv)
 {
     constructorInit();
@@ -126,7 +126,19 @@ CompositeViewer::~CompositeViewer()
         citr != contexts.end();
         ++citr)
     {
-        (*citr)->close();
+        osg::GraphicsContext* gc = *citr;
+
+        // Run destroy operation on each context before closing it
+        if (_cleanUpOperation.valid() && gc->valid())
+        {
+            gc->makeCurrent();
+
+            (*_cleanUpOperation)(gc);
+
+            gc->releaseContext();
+        }
+
+        gc->close();
     }
 
     OSG_INFO<<"finished CompositeViewer::~CompositeViewer()"<<std::endl;
@@ -245,34 +257,23 @@ bool CompositeViewer::isRealized() const
 
 bool CompositeViewer::checkNeedToDoFrame()
 {
+    // check if any event handler has prompted a redraw
     if (_requestRedraw) return true;
     if (_requestContinousUpdate) return true;
 
+    // check if any view needs to update the scene graph
     for(RefViews::iterator itr = _views.begin();
         itr != _views.end();
         ++itr)
     {
         osgViewer::View* view = itr->get();
-        if (view)
-        {
-            // check if the database pager needs to update the scene
-            if (view->getDatabasePager()->requiresUpdateSceneGraph() ||
-                view->getDatabasePager()->getRequestsInProgress()) return true;
-
-            // check if there are camera update callbacks
-            if (view->getCamera()->getUpdateCallback()) return true;
-
-            // check if there are node update callbacks
-            if (view->getSceneData() != 0)
-            {
-                if (view->getSceneData()->getUpdateCallback() || (view->getSceneData()->getNumChildrenRequiringUpdateTraversal()>0) ) return true;
-            }
-        }
+        if (view && (view->requiresUpdateSceneGraph() || view->requiresRedraw())) return true;
     }
 
     // check if events are available and need processing
     if (checkEvents()) return true;
 
+    // and check again if any event handler has prompted a redraw
     if (_requestRedraw) return true;
     if (_requestContinousUpdate) return true;
 
@@ -626,7 +627,6 @@ void CompositeViewer::realize()
     // attach contexts to _incrementalCompileOperation if attached.
     if (_incrementalCompileOperation) _incrementalCompileOperation->assignContexts(contexts);
 
-
     bool grabFocus = true;
     if (grabFocus)
     {
@@ -654,9 +654,6 @@ void CompositeViewer::realize()
 
     if (osg::DisplaySettings::instance()->getCompileContextsHint())
     {
-        int numProcessors = OpenThreads::GetNumberOfProcessors();
-        int processNum = 0;
-
         for(unsigned int i=0; i<= osg::GraphicsContext::getMaxContextID(); ++i)
         {
             osg::GraphicsContext* gc = osg::GraphicsContext::getOrCreateCompileContext(i);
@@ -664,10 +661,7 @@ void CompositeViewer::realize()
             if (gc)
             {
                 gc->createGraphicsThread();
-                gc->getGraphicsThread()->setProcessorAffinity(processNum % numProcessors);
                 gc->getGraphicsThread()->startThread();
-
-                ++processNum;
             }
         }
     }
@@ -759,7 +753,7 @@ void CompositeViewer::generateSlavePointerData(osg::Camera* camera, osgGA::GUIEv
         float y = event.getY();
 
         bool invert_y = event.getMouseYOrientation()==osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS;
-        if (invert_y && gw->getTraits()) y = gw->getTraits()->height - y;
+        if (invert_y && gw->getTraits()) y = gw->getTraits()->height - 1 - y;
 
         double master_min_x = -1.0;
         double master_max_x = 1.0;
@@ -772,13 +766,13 @@ void CompositeViewer::generateSlavePointerData(osg::Camera* camera, osgGA::GUIEv
             osg::Viewport* viewport = view_masterCamera->getViewport();
             master_min_x = viewport->x();
             master_min_y = viewport->y();
-            master_max_x = viewport->x()+viewport->width();
-            master_max_y = viewport->y()+viewport->height();
+            master_max_x = viewport->x() + viewport->width() - 1;
+            master_max_y = viewport->y() + viewport->height() - 1;
             masterCameraVPW *= viewport->computeWindowMatrix();
         }
 
         // slave Camera tahnks to sharing the same View
-        osg::View::Slave* slave = view ? view->findSlaveForCamera(camera) : 0;
+        osg::View::Slave* slave = view->findSlaveForCamera(camera);
         if (slave)
         {
             if (camera->getReferenceFrame()==osg::Camera::RELATIVE_RF && slave->_useMastersSceneData)
@@ -853,10 +847,10 @@ void CompositeViewer::generatePointerData(osgGA::GUIEventAdapter& event)
     float y = event.getY();
 
     bool invert_y = event.getMouseYOrientation()==osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS;
-    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - y;
+    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - 1 - y;
 
-    event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width,
-                                                    y, 0, gw->getTraits()->height));
+    event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width - 1,
+                                                    y, 0, gw->getTraits()->height - 1));
 
     event.setMouseYOrientationAndUpdateCoords(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
 
@@ -872,10 +866,10 @@ void CompositeViewer::generatePointerData(osgGA::GUIEventAdapter& event)
         if (camera->getAllowEventFocus() &&
             camera->getRenderTargetImplementation()==osg::Camera::FRAME_BUFFER)
         {
-            osg::Viewport* viewport = camera ? camera->getViewport() : 0;
+            osg::Viewport* viewport = camera->getViewport();
             if (viewport &&
                 x >= viewport->x() && y >= viewport->y() &&
-                x <= (viewport->x()+viewport->width()) && y <= (viewport->y()+viewport->height()) )
+                x < (viewport->x()+viewport->width()) && y < (viewport->y()+viewport->height()) )
             {
                 activeCameras.push_back(camera);
             }
@@ -888,10 +882,10 @@ void CompositeViewer::generatePointerData(osgGA::GUIEventAdapter& event)
 
     if (camera)
     {
-        osg::Viewport* viewport = camera ? camera->getViewport() : 0;
+        osg::Viewport* viewport = camera->getViewport();
 
-        event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/viewport->width()*2.0f-1.0f, -1.0, 1.0,
-                                                            (y-viewport->y())/viewport->height()*2.0f-1.0f, -1.0, 1.0));
+        event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/(viewport->width() - 1)*2.0f-1.0f, -1.0, 1.0,
+                                                            (y-viewport->y())/(viewport->height() - 1)*2.0f-1.0f, -1.0, 1.0));
 
         osgViewer::View* view = dynamic_cast<osgViewer::View*>(camera->getView());
         osg::Camera* view_masterCamera = view ? view->getCamera() : 0;
@@ -913,20 +907,21 @@ void CompositeViewer::reprojectPointerData(osgGA::GUIEventAdapter& source_event,
     float y = dest_event.getY();
 
     bool invert_y = dest_event.getMouseYOrientation()==osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS;
-    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - y;
+    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - 1 - y;
 
-    dest_event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width,
-                                                         y, 0, gw->getTraits()->height));
+    dest_event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width - 1,
+                                                         y, 0, gw->getTraits()->height - 1));
 
     dest_event.setMouseYOrientationAndUpdateCoords(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
 
-    osg::Camera* camera = (source_event.getNumPointerData()>=2) ? dynamic_cast<osg::Camera*>(source_event.getPointerData(1)->object.get()) : 0;
+    osg::Object* object = (source_event.getNumPointerData()>=2) ? source_event.getPointerData(1)->object.get() : 0;
+    osg::Camera* camera = object ? object->asCamera() : 0;
     osg::Viewport* viewport = camera ? camera->getViewport() : 0;
 
     if (!viewport) return;
 
-    dest_event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/viewport->width()*2.0f-1.0f, -1.0, 1.0,
-                                                             (y-viewport->y())/viewport->height()*2.0f-1.0f, -1.0, 1.0));
+    dest_event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/(viewport->width() - 1)*2.0f-1.0f, -1.0, 1.0,
+                                                             (y-viewport->y())/(viewport->height() - 1)*2.0f-1.0f, -1.0, 1.0));
 
     osgViewer::View* view = dynamic_cast<osgViewer::View*>(camera->getView());
     osg::Camera* view_masterCamera = view ? view->getCamera() : 0;
@@ -1004,13 +999,39 @@ void CompositeViewer::eventTraversal()
         osgGA::GUIEventAdapter* event = (*itr)->asGUIEventAdapter();
         if (!event) continue;
 
+        enum EventClassification
+        {
+            EVENT_FOR_VIEW_ASSOCIATED_WITH_FOCUS,
+            EVENT_FOR_VIEWS_ASSOCIATED_WITH_WINDOW,
+            EVENT_FOR_ALL_VIEWS
+        };
+
+        EventClassification classification = EVENT_FOR_ALL_VIEWS;
+
         switch(event->getEventType())
         {
+            case(osgGA::GUIEventAdapter::CLOSE_WINDOW):
+            case(osgGA::GUIEventAdapter::RESIZE):
+                classification = EVENT_FOR_VIEWS_ASSOCIATED_WITH_WINDOW;
+                break;
+
+            case(osgGA::GUIEventAdapter::QUIT_APPLICATION):
+            case(osgGA::GUIEventAdapter::USER):
+                classification = EVENT_FOR_ALL_VIEWS;
+                break;
+
+            case(osgGA::GUIEventAdapter::KEYDOWN):
+            case(osgGA::GUIEventAdapter::KEYUP):
+                classification = EVENT_FOR_VIEW_ASSOCIATED_WITH_FOCUS;
+                if (_previousEvent.valid()) event->copyPointerDataFrom(*_previousEvent);
+                break;
+
             case(osgGA::GUIEventAdapter::PUSH):
             case(osgGA::GUIEventAdapter::RELEASE):
             case(osgGA::GUIEventAdapter::DOUBLECLICK):
             case(osgGA::GUIEventAdapter::MOVE):
             case(osgGA::GUIEventAdapter::DRAG):
+            case(osgGA::GUIEventAdapter::SCROLL):
             {
                 if ((event->getEventType()!=osgGA::GUIEventAdapter::DRAG && event->getEventType()!=osgGA::GUIEventAdapter::RELEASE) ||
                     !_previousEvent ||
@@ -1026,27 +1047,30 @@ void CompositeViewer::eventTraversal()
 
                 _previousEvent = event;
 
+                classification = EVENT_FOR_VIEW_ASSOCIATED_WITH_FOCUS;
                 break;
             }
+
             default:
                 if (_previousEvent.valid()) event->copyPointerDataFrom(*_previousEvent);
                 break;
         }
 
         osgGA::PointerData* pd = event->getNumPointerData()>0 ? event->getPointerData(event->getNumPointerData()-1) : 0;
-        osg::Camera* camera = pd ? dynamic_cast<osg::Camera*>(pd->object.get()) : 0;
+        osg::Object* object = pd ? pd->object.get() : 0;
+        osg::Camera* camera = object ? object->asCamera() : 0;
         osgViewer::View* view = camera ? dynamic_cast<osgViewer::View*>(camera->getView()) : 0;
 
         if (!view)
         {
             if (_viewWithFocus.valid())
             {
-                // OSG_NOTICE<<"Falling back to using _viewWithFocus"<<std::endl;
+                OSG_INFO<<"Falling back to using _viewWithFocus"<<std::endl;
                 view = _viewWithFocus.get();
             }
             else if (!_views.empty())
             {
-                // OSG_NOTICE<<"Falling back to using first view as one with focus"<<std::endl;
+                OSG_INFO<<"Falling back to using first view as one with focus"<<std::endl;
                 view = _views[0].get();
             }
         }
@@ -1054,12 +1078,57 @@ void CompositeViewer::eventTraversal()
         // reassign view with focus
         if (_viewWithFocus != view)  _viewWithFocus = view;
 
-        if (view)
+        switch(classification)
         {
-            viewEventsMap[view].push_back( event );
+            case(EVENT_FOR_VIEW_ASSOCIATED_WITH_FOCUS):
+            {
+                if (view)
+                {
+                    OSG_INFO<<"Sending EVENT_FOR_VIEW_ASSOCIATED_WITH_FOCUS event "<<event<<" to view "<<view<<std::endl;
 
-            osgGA::GUIEventAdapter* eventState = view->getEventQueue()->getCurrentEventState();
-            eventState->copyPointerDataFrom(*event);
+                    viewEventsMap[view].push_back( event );
+
+                    osgGA::GUIEventAdapter* eventState = view->getEventQueue()->getCurrentEventState();
+                    eventState->copyPointerDataFrom(*event);
+                }
+                break;
+            }
+            case(EVENT_FOR_VIEWS_ASSOCIATED_WITH_WINDOW):
+            {
+                typedef std::set<View*> ViewSet;
+                ViewSet views;
+                osg::GraphicsContext* gc = event->getGraphicsContext();
+                if (gc)
+                {
+                    osg::GraphicsContext::Cameras& cameras = gc->getCameras();
+                    for(osg::GraphicsContext::Cameras::iterator citr = cameras.begin();
+                        citr != cameras.end();
+                        ++citr)
+                    {
+                        osgViewer::View* camera_view = dynamic_cast<osgViewer::View*>((*citr)->getView());
+                        if (camera_view) views.insert(camera_view);
+                    }
+                }
+                for(ViewSet::iterator vitr=views.begin();
+                    vitr!=views.end();
+                    ++vitr)
+                {
+                    OSG_INFO<<"Sending EVENT_FOR_VIEWS_ASSOCIATED_WITH_WINDOW event "<<event<<" to view "<<*vitr<<std::endl;
+                    viewEventsMap[*vitr].push_back( event );
+                }
+                break;
+            }
+            case(EVENT_FOR_ALL_VIEWS):
+            {
+                for(RefViews::iterator vitr=_views.begin();
+                    vitr!=_views.end();
+                    ++vitr)
+                {
+                    OSG_INFO<<"Sending EVENT_FOR_ALL_VIEWS event "<<event<<" to view "<<vitr->get()<<std::endl;
+                    viewEventsMap[vitr->get()].push_back( event );
+                }
+                break;
+            }
         }
 
         _previousEvent = event;
@@ -1209,13 +1278,13 @@ void CompositeViewer::eventTraversal()
                     osg::NodeVisitor::TraversalMode tm = _eventVisitor->getTraversalMode();
                     _eventVisitor->setTraversalMode(osg::NodeVisitor::TRAVERSE_NONE);
 
-                    if (view->getCamera() && view->getCamera()->getEventCallback()) view->getCamera()->accept(*_eventVisitor);
+                    if (view->getCamera()) view->getCamera()->accept(*_eventVisitor);
 
                     for(unsigned int i=0; i<view->getNumSlaves(); ++i)
                     {
                         osg::View::Slave& slave = view->getSlave(i);
                         osg::Camera* camera = view->getSlave(i)._camera.get();
-                        if (camera && slave._useMastersSceneData && camera->getEventCallback())
+                        if (camera && slave._useMastersSceneData)
                         {
                             camera->accept(*_eventVisitor);
                         }
@@ -1343,13 +1412,13 @@ void CompositeViewer::updateTraversal()
             osg::NodeVisitor::TraversalMode tm = _updateVisitor->getTraversalMode();
             _updateVisitor->setTraversalMode(osg::NodeVisitor::TRAVERSE_NONE);
 
-            if (view->getCamera() && view->getCamera()->getUpdateCallback()) view->getCamera()->accept(*_updateVisitor);
+            if (view->getCamera()) view->getCamera()->accept(*_updateVisitor);
 
             for(unsigned int i=0; i<view->getNumSlaves(); ++i)
             {
                 osg::View::Slave& slave = view->getSlave(i);
                 osg::Camera* camera = slave._camera.get();
-                if (camera && slave._useMastersSceneData && camera->getUpdateCallback())
+                if (camera && slave._useMastersSceneData)
                 {
                     camera->accept(*_updateVisitor);
                 }
