@@ -16,6 +16,7 @@
 
 #include <osg/DeleteHandler>
 #include <osg/io_utils>
+#include <osg/os_utils>
 #include <osg/TextureRectangle>
 #include <osg/TextureCubeMap>
 
@@ -33,6 +34,8 @@
 #include <osgViewer/config/SphericalDisplay>
 #include <osgViewer/config/PanoramicSphericalDisplay>
 #include <osgViewer/config/WoWVxDisplay>
+#include <osgViewer/config/SingleWindow>
+
 
 #include <sstream>
 #include <string.h>
@@ -61,6 +64,7 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
     arguments.getApplicationUsage()->addCommandLineOption("--clear-color <color>","Set the background color of the viewer in the form \"r,g,b[,a]\".");
     arguments.getApplicationUsage()->addCommandLineOption("--screen <num>","Set the screen to use when multiple screens are present.");
     arguments.getApplicationUsage()->addCommandLineOption("--window <x y w h>","Set the position (x,y) and size (w,h) of the viewer window.");
+    arguments.getApplicationUsage()->addCommandLineOption("--borderless-window <x y w h>","Set the position (x,y) and size (w,h) of a borderless viewer window.");
 
     arguments.getApplicationUsage()->addCommandLineOption("--run-on-demand","Set the run methods frame rate management to only rendering frames when required.");
     arguments.getApplicationUsage()->addCommandLineOption("--run-continuous","Set the run methods frame rate management to rendering frames continuously.");
@@ -101,6 +105,8 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
     osg::DisplaySettings::instance()->readCommandLine(arguments);
     osgDB::readCommandLine(arguments);
 
+    getCamera()->readCommandLine(arguments);
+
     std::string colorStr;
     while (arguments.read("--clear-color",colorStr))
     {
@@ -134,7 +140,14 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
     bool ss3d = false;
     bool wowvx20 = false;
     bool wowvx42 = false;
-    if ((wowvx20=arguments.read("--wowvx-20")) || (wowvx42=arguments.read("--wowvx-42")) || arguments.read("--wowvx"))
+
+    if (arguments.read("--borderless-window",x,y,width,height))
+    {
+        osg::ref_ptr<osgViewer::SingleWindow> sw = new osgViewer::SingleWindow(x, y, width, height, screenNum);
+        sw->setWindowDecoration(false);
+        apply(sw.get());
+    }
+    else if ((wowvx20=arguments.read("--wowvx-20")) || (wowvx42=arguments.read("--wowvx-42")) || arguments.read("--wowvx"))
     {
         osg::ref_ptr<WoWVxDisplay> wow = new WoWVxDisplay;
 
@@ -171,12 +184,12 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
 
         if (ss3d)
         {
-            setThreadingModel(SingleThreaded);
+            //setThreadingModel(SingleThreaded);
             setUpViewFor3DSphericalDisplay(radius, collar, screenNum, intensityMap.get());
         }
         else
         {
-            setThreadingModel(SingleThreaded);
+            //setThreadingModel(SingleThreaded);
             setUpViewForPanoramicSphericalDisplay(radius, collar, screenNum, intensityMap.get());
         }
     }
@@ -194,7 +207,7 @@ Viewer::Viewer(osg::ArgumentParser& arguments)
 }
 
 Viewer::Viewer(const osgViewer::Viewer& viewer, const osg::CopyOp& copyop):
-    osg::Object(true),
+    osg::Object(viewer, copyop),
     ViewerBase(viewer),
     View(viewer,copyop)
 {
@@ -238,7 +251,19 @@ Viewer::~Viewer()
         citr != contexts.end();
         ++citr)
     {
-        (*citr)->close();
+        osg::GraphicsContext* gc = *citr;
+
+        // Run destroy operation on each context before closing it
+        if (_cleanUpOperation.valid() && gc->valid())
+        {
+            gc->makeCurrent();
+
+            (*_cleanUpOperation)(gc);
+
+            gc->releaseContext();
+        }
+
+        gc->close();
     }
 
     //OSG_NOTICE<<"finish Viewer::~Viewer()"<<std::endl;
@@ -275,6 +300,7 @@ void Viewer::take(osg::View& rhs)
         _updateVisitor = rhs_viewer->_updateVisitor;
 
         _realizeOperation = rhs_viewer->_realizeOperation;
+        _cleanUpOperation = rhs_viewer->_cleanUpOperation;
         _currentContext = rhs_viewer->_currentContext;
 
 
@@ -287,6 +313,7 @@ void Viewer::take(osg::View& rhs)
         rhs_viewer->_updateOperations = 0;
         rhs_viewer->_updateVisitor = 0;
         rhs_viewer->_realizeOperation = 0;
+        rhs_viewer->_cleanUpOperation = 0;
         rhs_viewer->_currentContext = 0;
 
     }
@@ -362,17 +389,11 @@ bool Viewer::checkNeedToDoFrame()
     if (_requestRedraw) return true;
     if (_requestContinousUpdate) return true;
 
-    // check if the database pager needs to update the scene
-    if (getDatabasePager()->requiresUpdateSceneGraph() || getDatabasePager()->getRequestsInProgress()) return true;
+    // check if the view needs to update the scene graph
+    if (requiresUpdateSceneGraph()) return true;
 
-    // check if there are camera update callbacks
-    if (_camera->getUpdateCallback()) return true;
-
-    // check if there are node update callbacks
-    if (getSceneData() != 0)
-    {
-        if (getSceneData()->getUpdateCallback() || (getSceneData()->getNumChildrenRequiringUpdateTraversal()>0) ) return true;
-    }
+    // check if the view needs to be redrawn
+    if (requiresRedraw()) return true;
 
     // check if events are available and need processing
     if (checkEvents()) return true;
@@ -487,28 +508,26 @@ void Viewer::realize()
 
         // no windows are already set up so set up a default view
 
-        const char* ptr = 0;
-        if ((ptr = getenv("OSG_CONFIG_FILE")) != 0)
+        std::string value;
+        if (osg::getEnvVar("OSG_CONFIG_FILE", value))
         {
-            readConfiguration(ptr);
+            readConfiguration(value);
         }
         else
         {
             int screenNum = -1;
-            if ((ptr = getenv("OSG_SCREEN")) != 0)
-            {
-                if (strlen(ptr)!=0) screenNum = atoi(ptr);
-                else screenNum = -1;
-            }
+            osg::getEnvVar("OSG_SCREEN", screenNum);
 
             int x = -1, y = -1, width = -1, height = -1;
-            if ((ptr = getenv("OSG_WINDOW")) != 0)
-            {
-                std::istringstream iss(ptr);
-                iss >> x >> y >> width >> height;
-            }
+            osg::getEnvVar("OSG_WINDOW", x, y, width, height);
 
-            if (width>0 && height>0)
+            if (osg::getEnvVar("OSG_BORDERLESS_WINDOW", x, y, width, height))
+            {
+                osg::ref_ptr<osgViewer::SingleWindow> sw = new osgViewer::SingleWindow(x, y, width, height, screenNum);
+                sw->setWindowDecoration(false);
+                apply(sw.get());
+            }
+            else if (width>0 && height>0)
             {
                 if (screenNum>=0) setUpViewInWindow(x, y, width, height, screenNum);
                 else setUpViewInWindow(x,y,width,height);
@@ -596,9 +615,6 @@ void Viewer::realize()
 
     if (osg::DisplaySettings::instance()->getCompileContextsHint())
     {
-        int numProcessors = OpenThreads::GetNumberOfProcessors();
-        int processNum = 0;
-
         for(unsigned int i=0; i<= osg::GraphicsContext::getMaxContextID(); ++i)
         {
             osg::GraphicsContext* gc = osg::GraphicsContext::getOrCreateCompileContext(i);
@@ -606,10 +622,7 @@ void Viewer::realize()
             if (gc)
             {
                 gc->createGraphicsThread();
-                gc->getGraphicsThread()->setProcessorAffinity(processNum % numProcessors);
                 gc->getGraphicsThread()->startThread();
-
-                ++processNum;
             }
         }
     }
@@ -690,7 +703,7 @@ void Viewer::generateSlavePointerData(osg::Camera* camera, osgGA::GUIEventAdapte
         float y = event.getY();
 
         bool invert_y = event.getMouseYOrientation()==osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS;
-        if (invert_y && gw->getTraits()) y = gw->getTraits()->height - y;
+        if (invert_y && gw->getTraits()) y = gw->getTraits()->height - 1 - y;
 
         double master_min_x = -1.0;
         double master_max_x = 1.0;
@@ -703,8 +716,8 @@ void Viewer::generateSlavePointerData(osg::Camera* camera, osgGA::GUIEventAdapte
             osg::Viewport* viewport = getCamera()->getViewport();
             master_min_x = viewport->x();
             master_min_y = viewport->y();
-            master_max_x = viewport->x()+viewport->width();
-            master_max_y = viewport->y()+viewport->height();
+            master_max_x = viewport->x() + viewport->width() - 1;
+            master_max_y = viewport->y() + viewport->height() - 1;
             masterCameraVPW *= viewport->computeWindowMatrix();
         }
 
@@ -777,7 +790,6 @@ void Viewer::generateSlavePointerData(osg::Camera* camera, osgGA::GUIEventAdapte
     }
 }
 
-
 void Viewer::generatePointerData(osgGA::GUIEventAdapter& event)
 {
     osgViewer::GraphicsWindow* gw = dynamic_cast<osgViewer::GraphicsWindow*>(event.getGraphicsContext());
@@ -787,10 +799,10 @@ void Viewer::generatePointerData(osgGA::GUIEventAdapter& event)
     float y = event.getY();
 
     bool invert_y = event.getMouseYOrientation()==osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS;
-    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - y;
+    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - 1 - y;
 
-    event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width,
-                                                    y, 0, gw->getTraits()->height));
+    event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width - 1,
+                                                    y, 0, gw->getTraits()->height - 1));
 
     event.setMouseYOrientationAndUpdateCoords(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
 
@@ -808,10 +820,10 @@ void Viewer::generatePointerData(osgGA::GUIEventAdapter& event)
             camera->getAllowEventFocus() &&
             camera->getRenderTargetImplementation()==osg::Camera::FRAME_BUFFER)
         {
-            osg::Viewport* viewport = camera ? camera->getViewport() : 0;
+            osg::Viewport* viewport = camera->getViewport();
             if (viewport &&
                 x >= viewport->x() && y >= viewport->y() &&
-                x <= (viewport->x()+viewport->width()) && y <= (viewport->y()+viewport->height()) )
+                x < (viewport->x()+viewport->width()) && y < (viewport->y()+viewport->height()) )
             {
                 activeCameras.push_back(camera);
             }
@@ -824,10 +836,10 @@ void Viewer::generatePointerData(osgGA::GUIEventAdapter& event)
 
     if (camera)
     {
-        osg::Viewport* viewport = camera ? camera->getViewport() : 0;
+        osg::Viewport* viewport = camera->getViewport();
 
-        event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/viewport->width()*2.0f-1.0f, -1.0, 1.0,
-                                                            (y-viewport->y())/viewport->height()*2.0f-1.0f, -1.0, 1.0));
+        event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/(viewport->width() - 1)*2.0f-1.0f, -1.0, 1.0,
+                                                            (y-viewport->y())/(viewport->height() - 1)*2.0f-1.0f, -1.0, 1.0));
 
         // if camera isn't the master it must be a slave and could need reprojecting.
         if (camera!=getCamera())
@@ -846,20 +858,21 @@ void Viewer::reprojectPointerData(osgGA::GUIEventAdapter& source_event, osgGA::G
     float y = dest_event.getY();
 
     bool invert_y = dest_event.getMouseYOrientation()==osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS;
-    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - y;
+    if (invert_y && gw->getTraits()) y = gw->getTraits()->height - 1 - y;
 
-    dest_event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width,
-                                                         y, 0, gw->getTraits()->height));
+    dest_event.addPointerData(new osgGA::PointerData(gw, x, 0, gw->getTraits()->width - 1,
+                                                         y, 0, gw->getTraits()->height - 1));
 
     dest_event.setMouseYOrientationAndUpdateCoords(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
 
-    osg::Camera* camera = (source_event.getNumPointerData()>=2) ? dynamic_cast<osg::Camera*>(source_event.getPointerData(1)->object.get()) : 0;
+    osg::Object* object = (source_event.getNumPointerData()>=2) ? source_event.getPointerData(1)->object.get() : 0;
+    osg::Camera* camera = object ? object->asCamera() : 0;
     osg::Viewport* viewport = camera ? camera->getViewport() : 0;
 
     if (!viewport) return;
 
-    dest_event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/viewport->width()*2.0f-1.0f, -1.0, 1.0,
-                                                             (y-viewport->y())/viewport->height()*2.0f-1.0f, -1.0, 1.0));
+    dest_event.addPointerData(new osgGA::PointerData(camera, (x-viewport->x())/(viewport->width() - 1)*2.0f-1.0f, -1.0, 1.0,
+                                                             (y-viewport->y())/(viewport->height() - 1)*2.0f-1.0f, -1.0, 1.0));
 
     // if camera isn't the master it must be a slave and could need reprojecting.
     if (camera!=getCamera())
@@ -1071,13 +1084,13 @@ void Viewer::eventTraversal()
             osg::NodeVisitor::TraversalMode tm = _eventVisitor->getTraversalMode();
             _eventVisitor->setTraversalMode(osg::NodeVisitor::TRAVERSE_NONE);
 
-            if (_camera.valid() && _camera->getEventCallback()) _camera->accept(*_eventVisitor);
+            if (_camera.valid()) _camera->accept(*_eventVisitor);
 
             for(unsigned int i=0; i<getNumSlaves(); ++i)
             {
                 osg::View::Slave& slave = getSlave(i);
                 osg::Camera* camera = slave._camera.get();
-                if (camera && slave._useMastersSceneData && camera->getEventCallback())
+                if (camera && slave._useMastersSceneData)
                 {
                     camera->accept(*_eventVisitor);
                 }
@@ -1177,13 +1190,13 @@ void Viewer::updateTraversal()
         osg::NodeVisitor::TraversalMode tm = _updateVisitor->getTraversalMode();
         _updateVisitor->setTraversalMode(osg::NodeVisitor::TRAVERSE_NONE);
 
-        if (_camera.valid() && _camera->getUpdateCallback()) _camera->accept(*_updateVisitor);
+        if (_camera.valid()) _camera->accept(*_updateVisitor);
 
         for(unsigned int i=0; i<getNumSlaves(); ++i)
         {
             osg::View::Slave& slave = getSlave(i);
             osg::Camera* camera = slave._camera.get();
-            if (camera && slave._useMastersSceneData && camera->getUpdateCallback())
+            if (camera && slave._useMastersSceneData)
             {
                 camera->accept(*_updateVisitor);
             }
