@@ -526,7 +526,7 @@ void RayTracedTechnique::traverse(osg::NodeVisitor& nv)
     {
         if (_volumeTile->getDirty()) _volumeTile->init();
 
-        osgUtil::UpdateVisitor* uv = dynamic_cast<osgUtil::UpdateVisitor*>(&nv);
+        osgUtil::UpdateVisitor* uv = nv.asUpdateVisitor();
         if (uv)
         {
             update(uv);
@@ -536,7 +536,7 @@ void RayTracedTechnique::traverse(osg::NodeVisitor& nv)
     }
     else if (nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR)
     {
-        osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(&nv);
+        osgUtil::CullVisitor* cv = nv.asCullVisitor();
         if (cv)
         {
             cull(cv);
